package abstractFactory.hk;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel; 
import javax.swing.JButton; 
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Sets up and displays a new GUI given a concrete factory.
 *
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Display extends JFrame  { 
	private static final long serialVersionUID = 0L;

    /**
     * Sets up the frame with a label and a button created by the respective
     * concrete factories. Both button and frame receive their appropriate 
     * listeners to close the frame when either the button is clicked or
     * the frame is closing.
     *
     * @param factory the factory team to create GUI elements
     */
	public Display(final ComponentFactoryTeam factory) {
		super("New GUI");
		JLabel label = factory.createLabel(); 
		JButton button = factory.createButton("OK");
		button.addActionListener(new myActionListener(this));
		JPanel panel = new JPanel();
		panel.add(label);
		panel.add(button);
		this.getContentPane().add(panel);
		this.pack();
		this.setVisible(true); 
		this.addWindowListener(new myWindowListener(this));
	}

	/**
	 * Adds a window listener that closes the frame on demand 
	 */
	private class myWindowListener extends WindowAdapter {
		Display display = null;
		
		protected myWindowListener(Display display) {
			super();
			this.display = display;   
		}
		
		public void windowClosing(WindowEvent e) {
			display.setVisible(false);
		}
	}

	/**
	 * Adds a button listener that closes the frame on demand 
	 */
	private class myActionListener implements ActionListener {
	    Display display;
	    
	    protected myActionListener(Display display) {
	        super();
	        this.display = display;
	    }
	    
	    public void actionPerformed(ActionEvent e) { 
			display.setVisible(false);
		}
	}
}
