package abstractFactory.jcooper;

/**
 * Abstract Factory which returns one of three gardens
 * 
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class GardenFactory {
	private AbstractGardenFactoryTeam activeFactory = null;

	final private AbstractGardenFactoryTeam annualFactory = new AnnualGardenFactoryTeam();
	final private AbstractGardenFactoryTeam perennialFactory = new PerennialGardenFactoryTeam();
	final private AbstractGardenFactoryTeam vegetableFactory = new VegetableGardenFactoryTeam();

	public void setFactory(String gtype) {
		if (gtype.equals("Perennial"))
			activeFactory = perennialFactory;
		else if (gtype.equals("Annual"))
			activeFactory = annualFactory;
		else activeFactory = vegetableFactory; // default

		System.out.println("Garden factory set to " + gtype);
	}
	public boolean isActive() {
		return activeFactory != null;
	}
	public Plant makeShade(){
		return makeShade(activeFactory);
	}
	public Plant makeShade(final AbstractGardenFactoryTeam chosenFactory) {
		shadeCreator<@chosenFactory> shadeCreator = new shadeCreator<@chosenFactory>();
		return shadeCreator.makeShade();
	}

	public Plant makeCenter(){
		return makeCenter(activeFactory);
	}
	public Plant makeCenter(final AbstractGardenFactoryTeam chosenFactory) {
		centerCreator<@chosenFactory> centerCreator = new centerCreator<@chosenFactory>();
		return centerCreator.makeCenter();
	}

	public Plant makeBorder(){
		return makeBorder(activeFactory);
	}
	public Plant makeBorder(final AbstractGardenFactoryTeam chosenFactory) {
		borderCreator<@chosenFactory> borderCreator = new borderCreator<@chosenFactory>();
		return borderCreator.makeBorder();	
	}
}
