package abstractFactory.jcooper;

import java.awt.*;
import java.awt.event.*;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Window4AbstractFactory extends Frame {
	private static final long serialVersionUID = 0L;

	public Window4AbstractFactory(String name) {
		super(name);
		setLayout(new GridLayout(1, 2));
		Panel leftPanel = new Panel();
		Panel rightPanel = new Panel();
		add(leftPanel);
		add(rightPanel);

		// create label and 3 radio buttons on left side
		leftPanel.setLayout(new GridLayout(4, 1));
		leftPanel.add(new Label("Garden type"));
		CheckboxGroup group = new CheckboxGroup();
		Checkbox vegieCheckBox = new Checkbox("Vegetable", group, false);
		Checkbox annualCheckBox = new Checkbox("Annual", group, false);
		Checkbox perennialCheckBox = new Checkbox("Perennial", group, false);
		leftPanel.add(vegieCheckBox);
		leftPanel.add(annualCheckBox);
		leftPanel.add(perennialCheckBox);

		// now create right side
		rightPanel.setLayout(new GridLayout(2, 1));
		final WindowPanel gardenPanel = new WindowPanel();
		gardenPanel.setBackground(Color.WHITE);

		final GardenFactory gardenFactory = new GardenFactory();
		ItemGardenListener itemListener =
			new ItemGardenListener(gardenFactory, gardenPanel);
		vegieCheckBox.addItemListener(itemListener);
		perennialCheckBox.addItemListener(itemListener);
		annualCheckBox.addItemListener(itemListener);

		rightPanel.add(gardenPanel);
		Panel bottomRightPanel = new Panel();
		rightPanel.add(bottomRightPanel);

		Button centerButton = new Button("Central");
		Button borderButton = new Button("Border");
		Button shadeButton = new Button("Shade");
		Button quitButton = new Button("Quit");

		centerButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (gardenFactory.isActive())
					gardenPanel.setCenter(gardenFactory.makeCenter().getName());
				gardenPanel.repaint();
			}
		});

		borderButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (gardenFactory.isActive())
					gardenPanel.setBorder(gardenFactory.makeBorder().getName());
				gardenPanel.repaint();
			}
		});
		shadeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (gardenFactory.isActive())
					gardenPanel.setShade(gardenFactory.makeShade().getName());
				gardenPanel.repaint();
			}
		});
		quitButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});
		bottomRightPanel.add(shadeButton);
		bottomRightPanel.add(borderButton);
		bottomRightPanel.add(centerButton);
		bottomRightPanel.add(quitButton);
		setBackground(Color.GREEN);
		setBounds(200, 200, 400, 300);
		setVisible(true);
	}
}
