package adapter.hk;

/**
 * Implements the driver for the Adapter design pattern example.  
 *
 * Intent:  Convert the interface of a class into another interface clients
 * expect. Adapter lets classes work together that couldn't otherwise because
 * incompatible interfaces.
 *
 * Experimental setup: analoguous to the pattern structure as described on page
 * 141 of the "Design Patterns" book: 
 * 
 * Adaptee can print strings to System.out. 
 * Adapter allows to access Adaptee's functionality
 * via Target's interface.  
 *
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java 
 */
public class Main { 

    /**
     * the class to which Adaptee must be adapted
     */     	
	private static Writer myTarget; 

    /**
     * the Adaptee in the scenario
     */
	private static SystemOutPrinter adaptee; 

    /**
     * Implements the driver.  
     * 
     * @param args required for a main method, but ignored
     */
	public static void main(String[] args) {
		System.out.println("Creating the Adaptee...");
		adaptee = new SystemOutPrinter();

	    System.out.println("Creating the Adapter...");
		final PrinterAdapterTeam pat = new PrinterAdapterTeam();
		Adapter<@pat> adaptedClass = pat.getAdapted(adaptee);
		myTarget = adaptedClass;

		System.out.print  ("Adapter and Adaptee are the same object: ");
		System.out.println(myTarget.equals(adaptee));

		System.out.println("Issuing the request() to the Adapter...");
		myTarget.write("OT/J Test successful.");
	}
}
