/*
   4 de Out de 2010
 */
package bridge.hk;

/**
 * Implements the driver for the Bridge design pattern example.
 * 
 * Intent: Decouple an abstraction from its implementation so that the two can
 * vary independently.
 * 
 * Scenario: Have separate hierarchies for Abstractions (here: Screens) and
 * Implementors (here: ScreenImplementation), so that both can change
 * independently of each other Participants: Screen - Abstraction that defines
 * an interface for printing text and boxedText to System.out. GreetingScreen -
 * RefinedAbstraction that prints a boxed greeting message InformationScreen -
 * RefinedAbstraction that prints the system time (boxed) ScreenImplementation -
 * Implementor interface, defines basic operations to output formatted strings
 * StarImplementation - ConcreteImplementation that creates textBoxes of stars
 * CrossCapitalImplementation - ConcreteImplementation that creates textBoxes of
 * double crosses (hashes) and prints all text capitalized
 * 
 * The two different screens and screen implementations are tested in all
 * possible combinations.
 * 
 * @param args
 *            required by Java, but ignored
 */
public class Main {
	/**
	 * Implements the driver for this example. The two different screens and
	 * screen implementations are tested in all possible combinations.
	 * 
	 * @param args
	 *            required by Java, but ignored
	 */
	public static void main(String[] args) {
		System.out.println("Creating implementations...");
		ScreenImplementation i1 = new StarImplementation();
		ScreenImplementation i2 = new CrossCapitalImplementation();

		System.out.println("Creating abstraction (screens) / implementation combinations...");
		GreetingScreenTeam gs1 = new GreetingScreenTeam(i1);
		GreetingScreenTeam gs2 = new GreetingScreenTeam(i2);
		InformationScreenTeam is1 = new InformationScreenTeam(i1);
		InformationScreenTeam is2 = new InformationScreenTeam(i2);

		System.out.println("Starting test:\n");
		gs1.drawText("\nScreen 1 (Refined Abstraction 1, Implementation 1):");
		gs1.draw();
		gs2.drawText("\nScreen 2 (Refined Abstraction 1, Implementation 2):");
		gs2.draw();
		is1.drawText("\nScreen 3 (Refined Abstraction 2, Implementation 1):");
		is1.draw();
		is2.drawText("\nScreen 4 (Refined Abstraction 2, Implementation 2):");
		is2.draw();
	}
}
