package bridge.hk;

/**
 * Declares methods to print lines, decorators and text. Represents an
 * Implementor in the context of the Bridge design pattern.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract class ScreenImplementation {

	/**
	 * Prints a line feed.
	 * Default implementation: does it to System.out.
	 */
	public void printLine() {
		System.out.println();
	}

	/**
	 * Prints a decorator symbol (a string of length 1).
	 */
	abstract void printDecor();

	/**
	 * Prints the argument text.
	 * 
	 * @param text
	 *            the text to print
	 */
	abstract void printText(String text);
}
