package bridge.jcooper;

import org.objectteams.Team;

import java.util.Vector;

import jcooper_support.InputFile;
import jcooper_support.Support;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
	private static void printHelpMessage() {
		System.out.println("Design pattern Bridge");
		System.out.println();
		System.out.println("Intent: decouple an abstraction from its implementation so that the two");
		System.out.println("can vary independently.");
		System.out.println();
		System.out.println("The window that appears shows two lists with the same content.");
		System.out.println("Each list is a different implementation of a given functionality.");
		System.out.println("The functionality is to show the contents of file 'products.txt',");
		System.out.println("found in folder 'files'.");
		System.out.println("The view on the left is a \"simple list\". It discards the numeric");
		System.out.println("values associated with the names.");
		System.out.println("The view on the right is a \"table list\". It shows both the names");
		System.out.println("and the numeric values.");
	}

	private static void loadProductListFromInputFile(InputFile inputFile, Vector<String> products) {
		String str = inputFile.readLine();
		while (str != null) {
			products.addElement(str);
			str = inputFile.readLine();
		}
	}

	public static void main(String[] args) {
		printHelpMessage();
		InputFile inputFile = new InputFile(Support.FOLDER_LOCATION +  "\\" + "products.txt");
		Vector<String> products = new Vector<String>();
		loadProductListFromInputFile(inputFile, products);
		new BridgeTeam().activate(Team.ALL_THREADS);
		new Window4Bridge(products);
	}
}
