package bridge.jcooper;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

/**
 * MPM: In this scenario, this class may be the closest to an 'Abstraction'
 * participant in the Bridge pattern. One could argue that the two JPanels
 * objects, each containing one table, are the instances of 'Abstraction'.
 * 
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Window4Bridge extends JFrame {
	private static final long serialVersionUID = 0L;

	public Window4Bridge(Vector<String> products) {
		super("The Java Factory-- Products");
		setLF(); // set look and feel
		setCloseClick(); // set close on window close click

		JPanel mainPanel = new JPanel();
		getContentPane().add(mainPanel);
		mainPanel.setLayout(new GridLayout(1, 2));
		JPanel leftPanel = new JPanel();
		JPanel rightPanel = new JPanel();
		mainPanel.add(leftPanel);
		mainPanel.add(rightPanel);
		leftPanel.setLayout(new BorderLayout());
		rightPanel.setLayout(new BorderLayout());
		// add in customer view as list box
		leftPanel.add("North", new JLabel("Customer view"));
		JScrollPane productList = new ProductList(products);
		leftPanel.add("Center", productList);
		// add in execute view as table
		rightPanel.add("North", new JLabel("Executive view"));
		JScrollPane productTable = new ProductTable(products);
		rightPanel.add("Center", productTable);
		setSize(new Dimension(400, 300));
		setVisible(true);
	}

	private void setCloseClick() {
		// create window listener to respond to window close click
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
	}

	private void setLF() {
		// Force SwingApp to come up in the System look and feel
		String laf = UIManager.getSystemLookAndFeelClassName();
		try {
			UIManager.setLookAndFeel(laf);
		} catch (UnsupportedLookAndFeelException exc) {
			System.err.println("Warning: UnsupportedLookAndFeel: " + laf);
		} catch (Exception exc) {
			System.err.println("Error loading " + laf + ": " + exc);
		}
	}
}
