package builder.hk;

/**
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class AbstractCreatorTeam {

	public abstract class Builder {
		protected String representation;

		/**
		 * Defines a buildPart() operation for type parts.
		 * 
		 * @param type
		 *            the type to process
		 */
		public abstract void processType(String type);

		/**
		 * Defines a buildPart() operation for attribute parts.
		 * 
		 * @param newAttribute
		 *            the attribute to process
		 */
		public abstract void processAttribute(String newAttribute);

		/**
		 * Defines a buildPart() operation for value parts.
		 * 
		 * @param newValue
		 *            the value to process
		 */
		public abstract void processValue(String newValue);

		/**
		 * Defines a getResult() operation for Builders. This is the default
		 * implementation.
		 * 
		 * @return a representation of the build result
		 */
		public String getRepresentation() {
			return representation;
		}
	}

	protected String _product;
	protected Builder _builder;

	public AbstractCreatorTeam() {
		_builder = new Builder();
	}

	public void build() {
		_builder.processType("Person");
		_builder.processAttribute("Name");
		_builder.processValue("James Brick");
		_builder.processAttribute("Age");
		_builder.processValue("33");
		_builder.processAttribute("Occupation");
		_builder.processValue("Builder");
		_product = _builder.getRepresentation();
	}

	public String getProduct() {
		return _product;
	}
}
