package builder.hk;

/**
 * Implements the driver for the Builder design pattern example.
 *
 * Intent: Separate the construction of a complex object from its 
 * representation so that the same construction process can create different 
 * representations
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
    /**
     * Implements the driver for the Builder design pattern example.
     *
     * In this example, two different builders build string representations
     * of a person.
     * TextCreatorTeam creates a text-like representation, XMLCreatorTeam
     * a XML-like one.
     */
    public static void main(String[] args) {
        AbstractCreatorTeam builder1 = new TextCreatorTeam();
        AbstractCreatorTeam builder2 = new XMLCreatorTeam();

        builder1.build();
        builder2.build();

        System.out.println(builder1.getProduct());
        System.out.println(builder2.getProduct());
    }
}
