package builder.hk;

/**
 * Implements a ConcreteBuilder that builds XML descriptions of complex
 * objects consisting of type, attributes and values.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class XMLCreatorTeam extends AbstractCreatorTeam {
	public class Builder {
		 protected String type = null;
		 protected String attribute = null;

		/**
		 * Defines a buildPart() operation for type parts.
		 * 
		 * @param newType
		 *            the type to process
		 */
		public void processType(String newType) {
			representation = "<" + newType + ">\n";
			type = newType;
		}

		/**
		 * Defines a buildPart() operation for attribute parts.
		 * 
		 * @param newAttribute
		 *            the attribute to process
		 */
		public void processAttribute(String newAttribute) {
			if (attribute != null) {
				representation += ("</" + attribute + ">\n");
			}
			representation += ("\t<" + newAttribute + ">");
			this.attribute = newAttribute;
		}

		/**
		 * Defines a buildPart() operation for value parts.
		 * 
		 * @param newValue
		 *            the value to process
		 */
		public void processValue(String newValue) {
			representation += (newValue);
		}

		/**
		 * Defines the getResult() operation for Builders. Includes a
		 * default implementation.
		 */
		public String getRepresentation() {
			if (attribute != null) {
				representation += ("</" + attribute + ">\n");
				attribute = null;
			}
			if (type != null) {
				representation += ("</" + type + ">\n");
				type = null;
			}
			return representation;
		}
	}
}
