/*
   28 de Jun de 2010
 */
package builder.jcooper;

import java.awt.Checkbox;
import java.awt.GridLayout;
import java.awt.List;
import java.awt.Panel;
import java.util.Vector;

/**
 * This team plays the role of 'Director' in the pattern, as 'Director' is really
 * the context from which the 'Builder' participant works.
 *  
 * @author Miguel P. Monteiro
 */
public team class DirectorTeam {

	public abstract class Builder implements IBuilder {
		protected Vector<String> _choices;

		public Builder(Vector<String> choiceList) {
			_choices = choiceList; // save list
		}

		/**
		 * return a Panel of components
		 */
		public abstract Panel getUI();
		/**
		 * get list of items
		 */
		public abstract String[] getSelected(); 

		/**
		 * clear selections
		 */
		public abstract void clearAll();
	}

	protected class ListboxChoiceBuilder extends Builder {
		protected List list; // investment list goes here

		public ListboxChoiceBuilder(Vector<String> choices) {
			super(choices);
		}

		/**
		 * This class is a Builder return a Panel of components
		 */
		public Panel getUI() {
			// create a panel containing a list box
			Panel p = new Panel();
			list = new List(_choices.size()); // list box
			list.setMultipleMode(true); // multiple
			p.add(list);
			// add investments into list box
			for (int i = 0; i < _choices.size(); i++)
				list.addItem((String) _choices.elementAt(i));
			return p; // return the panel
		}

		/**
		 * get list of items
		 */
		public String[] getSelected() {
			int count = 0;
			// count the selected listbox lines
			for (int i = 0; i < list.getItemCount(); i++) {
				if (list.isIndexSelected(i))
					count++;
			}
			// create a string array big enough for those selected
			String[] slist = new String[count];
			// copy list elements into string array
			int j = 0;
			for (int i = 0; i < list.getItemCount(); i++) {
				if (list.isIndexSelected(i))
					slist[j++] = list.getItem(i);
			}
			return (slist);
		}

		public void clearAll() {
		}
	}

	protected class CheckBoxChoiceBuilder extends Builder {
		private int _count;
		private Panel _checkBoxPanel;

		public CheckBoxChoiceBuilder(Vector<String> choices) {
			super(choices);
			_count = 0;
			_checkBoxPanel = new Panel();
		}

		/**
		 * return a Panel of components
		 */
		public Panel getUI() {
			String s;
			// create a grid layout 1 column by n rows
			_checkBoxPanel.setLayout(new GridLayout(_choices.size(), 1));
			// and add labeled check boxes to it
			for (int i = 0; i < _choices.size(); i++) {
				s = (String) _choices.elementAt(i);
				_checkBoxPanel.add(new Checkbox(s));
				_count++;
			}
			return _checkBoxPanel;
		}

		/**
		 * Returns a list of selected items in the form of an array of String
		 */
		public String[] getSelected() {
			int selectedItemsCount = 0;
			// First count selected items
			for (int i = 0; i < _checkBoxPanel.getComponentCount(); i++)
				if (((Checkbox) _checkBoxPanel.getComponent(i)).getState())
					selectedItemsCount++;
			String[] selectedItems = new String[selectedItemsCount];
			int j = 0;
			// Copy selected items to array
			for (int i = 0; i < _checkBoxPanel.getComponentCount(); i++) {
				Checkbox checkbox = (Checkbox) _checkBoxPanel.getComponent(i);
				if (checkbox.getState())
					selectedItems[j++] = checkbox.getLabel();
			}
			return selectedItems;
		}

		public void clearAll() {
		}
	}

	public IBuilder getChoiceUI(Vector<String> choices) {
		IBuilder ui;
		if (choices.size() <= 3)
			// return a panel of checkboxes
			ui = new CheckBoxChoiceBuilder(choices);
		else
			// return a multi-select list box panel
			ui = new ListboxChoiceBuilder(choices);
		return ui;
	}
}
