/*
   28 de Jun de 2010
*/
package builder.jcooper;

import java.util.Vector;
/**
 * @author Miguel P. Monteiro
 */
public class Main {
	private static void printHelpMessage() {
		System.out.println("Design pattern Builder");
		System.out.println();
		System.out.println("Intent: Separate the construction of a complex object from its representation ");
		System.out.println("so that the same construction process can create different representations.");
		System.out.println();
		System.out.println("This example presents a window displaying lists of investments in each of three");
		System.out.println("categories: stocks, bonds and mutual funds.");
		System.out.println("The list at the left displays the three categories.");
		System.out.println("One list is displayed at a time, depending on which is selected.");
		System.out.println("Display adapts to whether the list has at most three, or more than three items.");
		System.out.println("If the list has three or less items, it is displayed as a set of check boxes.");
		System.out.println("If the list has more than three items, it is displayed as a multi-choice list.");
		System.out.println("Selected items are displayed in the console.");
		System.out.println();
		System.out.println("A builder is used to generate the adequate representation, depending on the");
		System.out.println("size of the list of selected items.");
		System.out.println("Participants:");
		System.out.println("\tDirector: ChoiceDirector");
		System.out.println("\tAbstract Builder: MultiChoiceAbstractBuilder");
		System.out.println("\tConcrete Builders: CheckBoxChoiceConcreteBuilder, ListboxChoiceConcreteBuilder");
		System.out.println("\tProduct: String array (selected items) & corresponding graphical representation (Panel)");
		System.out.println();
	}

	private static Vector<String> buildStocks() {
		Vector<String> stocks = new Vector<String>();
		stocks.add("Cisco");
		stocks.add("Coca Cola");
		stocks.add("General Electric");
		stocks.add("Harley Davidson");
		stocks.add("IBM");
		return stocks;
	}

	private static Vector<String> buildBonds() {
		Vector<String> mutuals = new Vector<String>();
		mutuals.add("Greek Government");
		mutuals.add("Spanish Government");
		mutuals.add("Portuguese Government");
		return mutuals;
	}

	private static Vector<String> buildMutuals() {
		Vector<String> mutualFunds = new Vector<String>();
		mutualFunds.addElement("Fidelity Magellan");
		mutualFunds.addElement("T Rowe Price");
		mutualFunds.addElement("Vanguard PrimeCap");
		mutualFunds.addElement("Lindner Fund");
		return mutualFunds;
	}

	public static void main(String[] args) {
		printHelpMessage();
		new Window4Builder(
			new DirectorTeam(),
			buildStocks(),
			buildBonds(),
			buildMutuals());
	}
}
