/*
   24 de Jun de 2010
 */
package builder.jcooper;

import java.util.Vector;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowListener;

/**
 * @author James Cooper - original conception
 * @author Miguel P. Monteiro, recreated from the partial description from Cooper's book.
 */
public class Window4Builder extends Frame {
	private static final long serialVersionUID = 0;

	private Button _plotButton, _exitButton;
	private List _stockList;
	private Panel _choicePanel;
	private Vector<String> _bonds, _stocks, _mutuals;
	private DirectorTeam _director; // the director
	private IBuilder _builder;

	public Window4Builder(DirectorTeam director, Vector<String> stocks, Vector<String> bonds, Vector<String> mutualFunds) {
		super("Wealth Builder"); // frame title bar
		setGUI(); // set up display
		_stocks = stocks;
		_bonds = bonds;
		_mutuals = mutualFunds;
		_director = director;
	}

	private void setGUI() {
		setLayout(new BorderLayout());
		Panel p = new Panel();
		add("Center", p);
		// center contains left and right panels
		p.setLayout(new GridLayout(1, 2));
		// left is list of stocks
		_stockList = new List(10);
		p.add(_stockList);
		_stockList.add("Stocks");
		_stockList.add("Bonds");
		_stockList.add("Mutual Funds");
		_stockList.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				stockList_Click();
				_plotButton.setEnabled(true);				
			}
		});
		// Plot button along bottom of display
		Panel southPanel = new Panel();
		southPanel.setBackground(Color.lightGray);
		add("South", southPanel);

		_plotButton = new Button("Plot");
		_plotButton.setEnabled(false); // disabled until stock picked
		_plotButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				String[] selectedInvestments = _builder.getSelected();
				for (int i = 0; i < selectedInvestments.length; i++)
					System.out.println(selectedInvestments[i]);
				System.out.println();
			}
		});
		southPanel.add(_plotButton);
		_exitButton = new Button("Quit");
		_exitButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				System.exit(0);
			}
		});
		southPanel.add(_exitButton);

		// right is empty at first
		_choicePanel = new Panel();
		_choicePanel.setBackground(Color.lightGray);
		p.add(_choicePanel);
		WindowListener w = new MyWindowAdapter(); // intercepts WindowClosing
		addWindowListener(w);

		setBounds(100, 100, 350, 200);
		setVisible(true);
	}

	private void stockList_Click() {
		Vector<String> investmentVector = null;
		int index = _stockList.getSelectedIndex();
		_choicePanel.removeAll(); // remove previous ui panel
		// this just switches among 3 different Vectors
		// and passes the one you select to the Builder pattern
		switch (index) {
		case 0:
			investmentVector = _stocks;
			break;
		case 1:
			investmentVector = _bonds;
			break;
		case 2:
			investmentVector = _mutuals;
		}
		_builder = _director.getChoiceUI(investmentVector); // get one of the UIs
		_choicePanel.add(_builder.getUI()); // insert in right panel
		_choicePanel.validate(); // re-layout and display
		_plotButton.setEnabled(true); // allow plots
	}
}
