package chain_of_responsibility.hk;

import javax.swing.JButton;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * A simple GUI button that implements its own ActionListener.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Button extends JButton {
	private static final long serialVersionUID = 0L;

	/**
	 * Creates a Button widget. An ActionListener is also added that calls the
	 * doClick(Click) method when the button is pressed
	 * 
	 * @param label
	 *            the button label
	 */
	public Button(String label) {
		super(label);
		this.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				doClick(new Click(ae));
			}
		});
	}

	/**
	 * An empty method that is called when the button is clicked. This method
	 * could also be defined in the concrete aspect.
	 * 
	 * @param click
	 *            the Click that was created when the button was
	 *            clicked.
	 */
	public void doClick(Click click) {
	}
}
