package chain_of_responsibility.hk;

import java.awt.event.ActionEvent;

/**
 * Represents the Request in the Chain of Responsibility pattern,
 * which is a button click in this case. Provides methods for accessing key
 * masks associated with the click (to find out whether the SHIFT, ALT, or CTRL
 * keys were pressed during the click).
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Click {

	/**
	 * the ActionEvent that describes this Click
	 */
	protected ActionEvent description;

	/**
	 * Creates a Click described by the provided ActionEvent.
	 * 
	 * @param description
	 *            the ActionEvent that describes this Click
	 */
	public Click(ActionEvent description) {
		this.description = description;
	}

	/**
	 * Convenience method for inquiring whether SHIFT was pressed while the
	 * click occured.
	 * 
	 * @return whether the SHIFT key was pressed when the click occured
	 */
	public boolean hasShiftMask() {
		return ((description.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
	}

	/**
	 * Convenience method for inquiring whether ALT was pressed while the
	 * click occured.
	 * 
	 * @return whether the ALT key was pressed when the click occured
	 */
	public boolean hasAltMask() {
		return ((description.getModifiers() & ActionEvent.ALT_MASK) != 0);
	}

	/**
	 * Convenience method for inquiring whether CTRL was pressed while the click
	 * occured.
	 * 
	 * @return whether the CTRL key was pressed when the click occured
	 */
	public boolean hasCtrlMask() {
		return ((description.getModifiers() & ActionEvent.CTRL_MASK) != 0);
	}
}
