package chain_of_responsibility.hk;

import chain_of_responsibility.ChainOfResponsibilityProtocol;

/**
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class ClickChainTeam extends ChainOfResponsibilityProtocol {

	protected class ButtonHandler extends Handler playedBy Button {
		public void handleDoClick(Click clickRequest) {
			receiveRequest(this, new Request(clickRequest));
		}
		void handleDoClick(Click c) <- after void doClick(Click c);

		public boolean acceptRequest(Request request) {
	    	System.out.println("Button is asked to accept the request...");
			return request.hasShiftMask(); 
	    }
		public void handleRequest(Request request) {
			System.out.println("Button is handling the event.\n");
		}
	}

	protected class PanelHandler extends Handler playedBy Panel {
		public boolean acceptRequest(Request request) {
	    	System.out.println("Panel is asked to accept the request...");
			return request.hasCtrlMask();
	    }
		public void handleRequest(Request request) {
			System.out.println("Panel is handling the event.\n");
		}
	}

	public class FrameHandler extends Handler playedBy Frame {
		public void handleRequest(Request request) {
			System.out.println("Frame is handling the event.\n");
		}
		public boolean acceptRequest(Request request) {
	    	System.out.println("Frame is asked to accept the request...");
			return request.hasAltMask();
	    } 
	}

	protected class Request implements ILowerable playedBy Click {
		public abstract boolean hasAltMask();
		hasAltMask -> hasAltMask;
		public abstract boolean hasCtrlMask();
		hasCtrlMask -> hasCtrlMask;
		public abstract boolean hasShiftMask();
		hasShiftMask -> hasShiftMask;
	}

	public <AnyBase base Handler, AnyBase1 base Handler>
	void setSucessor(AnyBase as Handler handler, AnyBase1 as Handler successor) {
		super.setSuccessor(handler, successor);
	}
}
