package chain_of_responsibility.jcooper;

import java.awt.Color;

import javax.swing.JTextField;

import chain_of_responsibility.ChainOfResponsibilityProtocol;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class CoRTeam extends ChainOfResponsibilityProtocol {

	public <AnyBase base Handler, AnyBase1 base Handler>
	void setSucessor(AnyBase as Handler handler, AnyBase1 as Handler successor) {
		super.setSuccessor(handler, successor);
	}

	protected class ImageHandler extends Handler playedBy Imager {
		void loadImage(String fileName) -> void loadImage(String fileName);
		boolean findImage(String fileName) -> boolean findImage(String fileName);

		public boolean acceptRequest(Request request) {
			return findImage(((myString)request.lower()).getString());
		}

		public void handleRequest(Request request) {
			String fileName =  ((myString)request.lower()).getString();
			loadImage(fileName);
		}
	}

	protected class ColorImageHandler extends Handler playedBy ColorImage {
		void setBackground(Color arg0) -> void setBackground(Color arg0);
		void repaint() -> void repaint();
		Color getColor(String mesg) -> Color getColor(String mesg);

		public boolean acceptRequest(Request request) {
			Color c = getColor(((myString)request.lower()).getString());
			return (c != null);
		}
		
		public void handleRequest(Request request) {
			String mesg =  ((myString)request.lower()).getString() ;
			Color c = getColor(mesg);
			setBackground(c);
			repaint();
		}
	}

	protected class SenderHandler extends Handler playedBy Sender {
		JTextField getTextField() -> get JTextField tx;

		public void handleRequest(Request request) {}
		
		public void handleActionPerformed() {
			String file = getTextField().getText();
			Handler succInChain = getSuccessor(this);
			if ((file.length() > 0) && (succInChain != null))
				receiveRequest(succInChain, new Request(new myString(file)));
		}
		handleActionPerformed <- after actionPerformed;
	}

	protected class RestListHandler extends Handler implements ILowerable playedBy RestList {
		void repaint() -> void repaint();

		public boolean acceptRequest(Request request) {
			return true;
		}
		
		public void handleRequest(Request request) {
			((RestList)this.lower()).add(((myString)request.lower()).getString()); //this is the end of the chain
			repaint();
		}
	}

	protected class FileListHandler extends Handler playedBy FileList {
		void setSelectedIndex(int i) -> void setSelectedIndex(int i);

		String[] getFiles() -> get String[] _files;
		//Personal decision to define 0 as default position
		public int selectedFilePos = 0;
		
		public boolean foundFile(myString file) {
			
		   boolean found = false;
		   int i = 0; 
		   while ((! found) && (i < getFiles().length))
		   {   
		      XFile xfile = new XFile(getFiles()[i]);
		      found = xfile.matchRoot(file.getString());
		      if (! found) i++;
		   }
		   if(found) selectedFilePos = i;
		   return found;
		}

		public boolean acceptRequest(Request request) {
			return foundFile((myString)request.lower());
		}
		
		public void handleRequest(Request request) {
			 setSelectedIndex(selectedFilePos);
			 selectedFilePos = 0;
		}
	}

	public class Request implements ILowerable playedBy myString {}
}
