package chain_of_responsibility.jcooper;

import java.awt.*;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;
import java.io.File;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Imager extends JPanel {
	private static final long serialVersionUID = 0L;

	private String _filePath;
	private Image _image;
	private boolean _loaded;

	public Imager(String filePath) {
		super();
		_filePath = filePath;
		_loaded = false;
		setBorder(new BevelBorder(BevelBorder.RAISED));
	}

	public void paint(Graphics g) {
		if (_loaded) {
			g.drawImage(_image, 0, 0, this);
		}
	}

	private void loadImage(String fileName) {
		// add complete path and jpg extension
		String imageFileName = _filePath + "\\" + fileName + ".jpg";
		_loaded = false;
		MediaTracker tracker = new MediaTracker(this);
		_image = Toolkit.getDefaultToolkit().getImage(imageFileName);
		tracker.addImage(_image, 0); // watch for image loading
		// this begins actual image loading
		try {
			tracker.waitForID(0, 1);
		} catch (InterruptedException e) {
		}
		_loaded = true;
		validate();
		repaint();
	}

	private boolean findImage(String fileName) {
		boolean found = false;
		XFile xfile = null;
		// File dir = new File(System.getProperty("user.dir"));
		File dir = new File(_filePath);
		String files[] = dir.list();
		int i = 0;
		while ((!found) && (i < files.length)) {
			xfile = new XFile(files[i]);
			found = xfile.matchRoot(fileName);
			if (!found)
				i++;
		}
		if (found) {
			System.out.println(fileName + " found");
			return xfile.matchName(fileName + ".jpg");
		} else
			System.err.println("\"" + fileName + "\" not found");
		return found; // false
	}
}
