package chain_of_responsibility.jcooper;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

import jcooper_support.JxFrame;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Window4CoR extends JxFrame {
	private static final long serialVersionUID = 0L;

	// list of chain members
	private Sender _sender; // gets commands
	private Imager _imager; // displays images
	private FileList _fileList; // highlights file names
	private ColorImage _colorImage; // shows colors
	private RestList _restList; // shows rest of list

	public Window4CoR(String filePath, CoRTeam chainTeam) {
		super("Chain demo");
//		CoRTeam ct = new CoRTeam();
//		ct.activate(Team.ALL_THREADS);
		JPanel jp = new JPanel();
		jp.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(jp);
		jp.setLayout(new GridLayout(1, 3));
		JPanel left = new JPanel();
		jp.add(left);
		left.setLayout(new GridLayout(2, 1));

		// create send panel and image display panel
		_sender = new Sender();
		_imager = new Imager(filePath);
		left.add(_imager);
		left.add(_sender);

		// create file list and color panels
		JPanel mid = new JPanel();
		jp.add(mid);
		mid.setBorder(new EmptyBorder(5, 5, 5, 5));
		mid.setLayout(new GridLayout(2, 1));
		_fileList = new FileList(filePath);
		_colorImage = new ColorImage();
		mid.add(_fileList);
		mid.add(_colorImage);

		// put list for rest of commands in right panel
		_restList = new RestList();
		jp.add(_restList);

		// set up the chain of responsibility
		chainTeam.setSucessor(_sender, _imager);
		chainTeam.setSucessor(_imager, _colorImage);
		chainTeam.setSucessor(_colorImage, _fileList);
		chainTeam.setSucessor(_fileList, _restList);

		// set up visual shape
		setSize(new Dimension(500, 300));
		setVisible(true);
	}
}
