package command;

import java.util.WeakHashMap;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class CommandProtocol {

	public abstract class Invoker {
	}
	public abstract class Receiver {
	}

	public abstract class Command {
		/*
		 * To be bound to a base method. Parameter "Object o" is the object
		 * requesting the ICommand execution
		 */
		// parameter type below should be Invoker but the compiler throws an
		// exception if we do this
		public abstract void execute(Object /*Invoker*/ invoker);
	}

	// //////////////////////////////
	// Invoker -> ICommand mapping //
	// //////////////////////////////

	/**
	 * stores the mapping between CommandInvokers and Commands
	 */
	private WeakHashMap<Invoker, Command> mappingInvokerToCommand = new WeakHashMap<Invoker, Command>();

	/**
	 * Sets a new command for an invoker
	 */
	public void setCommand(Invoker invoker, Command command) {
		mappingInvokerToCommand.put(invoker, command);
	}

	/**
	 * Returns the command for an invoker
	 */
	public Command getCommand(Invoker invoker) {
		return mappingInvokerToCommand.get(invoker);
	}

	// ///////////////////////////////
	// ICommand -> Receiver mapping //
	// ///////////////////////////////

	/**
	 * stores the mapping between Commands and Receivers
	 */
	private WeakHashMap<Command, Receiver> mappingCommandToReceiver = new WeakHashMap<Command, Receiver>();

	/**
	 * Sets a new receiver for a command
	 */
	public Receiver setReceiver(Command command, Receiver receiver) {
		return mappingCommandToReceiver.put(command, receiver);
	}

	/**
	 * Returns the receiver for a particular command
	 */
	public Receiver getReceiver(Command command) {
		return mappingCommandToReceiver.get(command);
	}
}
