package command.hk;

import javax.swing.JButton;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Implements a simple extension of JButton that supplies its own ActionListener
 * and calls its own clicked() method whenever the button is
 * pressed. This method calls the 
 * executeCommand() method on the button's
 * associated ICommand object.
 *
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java 
 */
public class Button extends JButton {
	private static final long serialVersionUID = 0L;

	/**
	 * Creates a new button with the provided label
	 * 
	 * @param label
	 *            the label of the button
	 */
	public Button(String label) {
		super(label);
		this.setActionCommand(label);
		this.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				clicked();
			}
		});
	}

	/**
	 * Calls executeCommand() on the associated command object.
	 * This method gets called whenever the button is pressed.
	 */
	public void clicked() {
	}

}