package command.hk;
 
import org.objectteams.Team;
import javax.swing.JFrame; 
import javax.swing.JPanel;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * Implements the driver for the command design pattern example. 
 *
 * Intent: Encapsulate a request as an object, thereby letting you 
 * parameterize clients with different requests, queue or log requests, 
 * and support undoable operations.
 *
 * Participating objects are Buttons as Invokers,
 * and a ButtonCommand and ButtonCommand2 as 
 * two ConcreteCommands.
 * CommandTeam defines which actions on Buttons trigger Commands.
 *
 * This example creates a simple GUI with three buttons. Each button has a 
 * command associated with it that is executed when the button is pressed. 
 * Button1 and button3 have the same command, button2 has a different one.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {

    /**
     * This example creates a simple GUI with three buttons. Each button has a 
     * command associated with it that is executed when the button is pressed. 
     * Button1 and button3 have the same command, button2 has a different one.
     */
	public static void main(String[] args) {
    	Button button1 = new Button("Print Date");
    	Button button2 = new Button("ICommand 1");
    	Button button3 = new Button("ICommand 2");

    	AbstractCommand com1 = new ButtonCommand();
    	AbstractCommand com2 = new ButtonCommand2();

    	CommandTeam ct = new CommandTeam();
    	ct.activate(Team.ALL_THREADS);
    	ct.setCommand(button1, com1);
    	ct.setCommand(button2, com2);
    	ct.setCommand(button3, com1);

		JPanel pane = new JPanel();
		pane.add(button1);
		pane.add(button2);
		pane.add(button3);

        JFrame frame = new JFrame("ICommand Pattern Example");
		frame.getContentPane().add(pane);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {System.exit(0);}
		});  
		frame.pack();
		frame.setVisible(true);
	}
}
