package composite;

import java.util.Enumeration;
import java.util.Vector;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class CompositeProtocol {

	protected abstract class Component {
	    /**
	     * Stores the children for this Directory (files and subdirectories)
	     */
		protected Vector<Component> children = new Vector<Component>();

		protected abstract void operation();
		protected void add(Component o) {
			this.children.add(o);
		}

		protected void remove(Component o) {
			this.children.remove(o);
		}

		protected Enumeration<Component> getChildren() {
			return children.elements();
		}
	}

	protected abstract class Composite extends Component {
		protected void operation() {
			for (int i = 0; i < children.size(); i++)
				children.elementAt(i).operation();
		}
	}

	protected abstract class Leaf extends Component {
	}

	public void addChild(Component parent, Component child) {
	// este Guard PRedicate nao faz nada, provavelmente pk este metodo e'
	// overridden na classe concreta
	// when (!CompositePattern.this.hasRole(parent, Leaf.class))
		parent.add(child);
	}

	public void addLeafChild(Composite parent, Leaf child) {
		parent.add(child);
	}

	public Enumeration<Component> getChildren(Composite parent) {
		return parent.getChildren();
	}

	public void Operation(Component c) {
		c.operation();
	}

	public void removeChild(Composite parent, Composite child) {
		// TODO
	}
}
