/*
   8 de Out de 2010
*/
package composite.hk2;

import java.util.Enumeration;
import java.util.Vector;

/**
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - initially adapted to Object Teams/Java
 * @author Miguel P. Monteiro - 2nd and subsequent OT/J versions
 */
public team class DirectoryTeam implements IComponent {
    /**
     * Stores the children for this Directory (files and subdirectories)
     */
	protected Vector<IComponent> _children = new Vector<IComponent>();
    /**
     * stores the name of this Directory
     */
	protected String name;

	protected class Leaf implements IComponent playedBy File {
		public abstract String getName();
		public abstract int getBaseSize();
		getName -> get name;
		getBaseSize -> get size;
		public String operation() {
			return toString();
		}
		public String toString() {
			return "File: " + getName() + " (" + getBaseSize() + " KB)";
		}

		/* 
		 * The operations below are dummies. They are not applicable to
		 * leaf objects but the pattern requires them to be part of the
		 * exposed interface (IComponent).
		 */
		public void addChild(IComponent o) {}
		public void remove(IComponent o) {}
		public IComponent getChild(int index) {
			return null;
		}
		public int getChildCount() {
			return 0;
		}
		public Enumeration<IComponent> getChildren() {
			return null;
		}
	}

    /**
     * Creates a new Directory with a given name
     *
     * @param name the name for the new Directory object
     */
	public DirectoryTeam(String name) {
		this.name = name;
	}

	/**
	 * Note: method addChild() below requires two overloaded implementations
	 * due to limitation in OT/J (at this time) preventing playedBy on Java
	 * interfaces.
	 */
	public void addChild(IComponent o) {
		_children.add((IComponent) o);
	}
	public void addChild(File as Leaf o) {
		_children.add((IComponent) o);
	}

    /**
     * Removes a child from the component
     * @param component the child to remove
     */
	public void remove(IComponent o) {
		_children.remove(o);
	}

	public Enumeration<IComponent> getChildren() {
		return _children.elements();
	}

    /**
     * Returns the number of chilren this Directory has
     * @returns the number of children of this Directory
     */
	public int getChildCount() {
		return _children.size();
	}

    /**
     * Returns a child of the Directory at the given position
     * @param index the position of the child
     */
	public IComponent getChild(int index) {
		return _children.elementAt(index);
	}

	/**
	 * Returns the size of this Directory. For simplicity, we define that only
	 * files have a tangible size, so this method returns 0.
	 * 
	 * @return the size of the component (on disk)
	 */
	public int getSize() {
		return 0;
	}
    /**
     * Overwrites the toString() method from Object
     * to print information about this Directory
     */
	public String toString() {
		return ("Directory: "+name);
	}
}
