/*
   8 de Out de 2010
 */
package composite.hk2;

/**
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 * @author Miguel P. Monteiro - 2nd and subsequent OT/J versions
 */
public class Main {

	/**
	 * helper variable to store recursion depth for pretty printing
	 */
	private static int indent = 0;

	/**
	 * Prints a number of spaces according to the current recursion depth
	 */
	private static void indent() {
		for (int i = 0; i < indent; i++)
			System.out.print(" ");
	}

	/**
	 * Pretty-prints a recursive composite structure
	 * 
	 * @param comp
	 *            the component denoting the entry point into the structure
	 */
	private static void printStructure(IComponent composite) {
		indent();
		System.out.println(composite);
		indent += 4;
		for (int i = 0; i < composite.getChildCount(); i++) {
			IComponent leaf = composite.getChild(i);
			printStructure(leaf);
		}
		indent -= 4;
	}

	/**
	 * This example creates a simple structure as follows: Composite directory1
	 * has three children: file1, directory2, and file3. directory2 has file2 as
	 * a child.
	 */
	public static void main(String[] args) {
		System.out
				.println("\n<<< Sample (revised) OT/J implementation of Composite pattern >>>\n");
		System.out.print("Creating Composite structure ...\n");

		DirectoryTeam directory1 = new DirectoryTeam("Directory1");
		DirectoryTeam directory2 = new DirectoryTeam("Directory2");
		File file1 = new File("File1", 123);
		File file2 = new File("File2", 4556);
		File file3 = new File("File3", 16);

		directory1.addChild(file1);
		directory1.addChild(directory2);
		directory2.addChild(file2);
		directory1.addChild(file3);

		System.out.println("done.");
		System.out.println("This is the Structure:");
		printStructure(directory1);
		System.out.println("\n<<< Test completed >>>");
	}

}
