package composite.jcooper;

import composite.CompositeProtocol;
import java.util.Enumeration;
import java.util.Vector;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class EmployeeTreeTeam extends CompositeProtocol {

	//This definition must be placed here for lowering to Components be possible
	protected abstract class Component implements ILowerable {}

	//This Guard Predicate should be in the reusable module
	//it is used to guarantee that a leaf instance is not lifted to a composite
	protected class Composite implements ILowerable playedBy Employee
	base when (!EmployeeTreeTeam.this.hasRole(base,Leaf.class)) {
		float getSalary() -> get float salary;
		protected void operation(){}
		callin float getSalaries() {
			float sum = getSalary();
			for(int i = 0; i <  this.children.size(); i++)
				sum += ((Employee)((Component)children.elementAt(i)).lower()).getSalary();
			return sum;
		}
		getSalaries <- replace getSalary;

		protected abstract String name();
		name -> getName;
	}

	protected class Leaf implements ILowerable playedBy Employee {
		protected void operation() {}

		abstract String name();
		name -> getName;
	}

	//shouldn't be always composite, leaf as well
	public void addChild(Employee as Composite parent, Employee as Composite child) {	
		parent.add(child);
	}

	public void addLeafChild(Employee as Composite parent, Employee as Leaf child)  {	
		parent.add(child);
	}

	//Returns an Enumeration de Employees instead of Enumeration de components
	//Needed because method is called from team
	public Enumeration<Employee> getChildren(Employee as Composite parent){
		Vector<Employee> res = new Vector<Employee>();
		Enumeration<Component> children = parent.getChildren();
		while(children.hasMoreElements()){
			res.add( ((Employee)((Component)children.nextElement()).lower()) );
		}

		return res.elements();
	}

	//Returns the Employee with a given name
	//(also requires explicit lowering)
	public Employee getChild(Employee as Composite parent, String child){
		Employee newEmp = null;
		if (parent.name().equals(child))
			return (Employee)parent.lower();
		boolean found = false;
		Enumeration<Component> e = parent.getChildren();
		while (e.hasMoreElements() && (!found)) {
			newEmp = (Employee)((Component)e.nextElement()).lower();
			found = newEmp.getName().equals(child);
			if (!found) {
				newEmp = this.getChild(newEmp,child);
				found = (newEmp != null);
			}
		}
		if (found)
			return newEmp;
		return null;
	}
}
