package composite.jcooper;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.Enumeration;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.border.BevelBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import jcooper_support.JxFrame;


/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Window4Composite extends JxFrame implements TreeSelectionListener {
	private static final long serialVersionUID = 0L;

	private JScrollPane sp;
	private JPanel treePanel;
	private JTree tree;
	private DefaultMutableTreeNode troot;
	private JLabel cost;

	private Employee boss = new Employee("CEO", 200000);
	private Employee marketVP, prodVP;
	private Employee salesMgr, advMgr;
	private Employee prodMgr, shipMgr;
	private EmployeeTreeTeam et;

	public Window4Composite(EmployeeTreeTeam et) {
		super("Employee tree");
		this.et = et;
		makeEmployees(et);
		setGUI();
	}

	private void setGUI() {
		treePanel = new JPanel();
		getContentPane().add(treePanel);
		treePanel.setLayout(new BorderLayout());
		sp = new JScrollPane();
		treePanel.add("Center", sp);
		treePanel.add("South", cost = new JLabel("          "));
		treePanel.setBorder(new BevelBorder(BevelBorder.RAISED));
		troot = new DefaultMutableTreeNode(boss.getName());
		tree = new JTree(troot);
		tree.setBackground(Color.lightGray);
		loadTree();
		// Put the Tree in a scroller
		sp.getViewport().add(tree);
		setSize(new Dimension(200, 300));
		setVisible(true);
	}

	public void loadTree() {
		DefaultMutableTreeNode troot;
		troot = new DefaultMutableTreeNode(boss.getName());
		treePanel.remove(tree);
		tree = new JTree(troot);
		tree.addTreeSelectionListener(this);
		sp.getViewport().add(tree);
		addNodes(troot, boss);
		tree.expandRow(0);
		repaint();
	}

	private void addNodes(DefaultMutableTreeNode pnode, Employee emp) {
		DefaultMutableTreeNode node;
		Enumeration e = et.getChildren(emp);
		while (e.hasMoreElements()) {
			Employee newEmp = (Employee) e.nextElement();
			node = new DefaultMutableTreeNode(newEmp.getName());
			pnode.add(node);
			addNodes(node, newEmp);
		}
	}

//	private void makeEmployees0(EmployeeTreeTeam et) {
//		et.addChild(boss, marketVP = new Employee("Marketing VP", 100000));
//		et.addChild(boss, prodVP = new Employee("Production VP", 100000));
//		et.addChild(marketVP, salesMgr = new Employee("Sales Mgr", 50000));
//		et.addChild(marketVP, advMgr = new Employee("Advt Mgr", 50000));
//		for (int i = 0; i < 5; i++)
//			et.addChild(salesMgr, new Employee("Sales "
//					+ new Integer(i).toString(), 30000.0F + (float) (Math
//					.random() - 0.5) * 10000));
//		et.addChild(advMgr, new Employee("Secy", 20000));
//		et.addChild(prodVP, prodMgr = new Employee("Prod Mgr", 40000));
//		et.addChild(prodVP, shipMgr = new Employee("Ship Mgr", 35000));
//		for (int i = 0; i < 4; i++)
//			et.addChild(prodMgr, new Employee("Manuf "
//					+ new Integer(i).toString(), 25000.0F + (float) (Math
//					.random() - 0.5) * 5000));
//		for (int i = 0; i < 3; i++)
//			et.addChild(shipMgr, new Employee("ShipClrk "
//					+ new Integer(i).toString(), 20000.0F + (float) (Math
//					.random() - 0.5) * 5000));
//	}

	private void makeEmployees(EmployeeTreeTeam et) {
		et.addChild(boss, marketVP = new Employee("Marketing VP", 60000));
		et.addChild(boss, prodVP = new Employee("Production VP", 60000));
		et.addChild(marketVP, salesMgr = new Employee("Sales Mgr", 30000));
		et.addChild(marketVP, advMgr = new Employee("Advt Mgr", 50000));
		for (int i = 0; i < 5; i++)
			et.addChild(salesMgr, new Employee("Sales "
					+ new Integer(i).toString(), 10000.0F));
		et.addChild(advMgr, new Employee("Secy", 30000));
		et.addChild(prodVP, prodMgr = new Employee("Prod Mgr", 20000));
		et.addChild(prodVP, shipMgr = new Employee("Ship Mgr", 10000));
		for (int i = 0; i < 4; i++)
			et.addChild(prodMgr, new Employee("Manuf "
					+ new Integer(i).toString(), 15000.0F));
		for (int i = 0; i < 3; i++)
			et.addChild(shipMgr, new Employee("ShipClrk "
					+ new Integer(i).toString(), 10000.0F));
	}

	public void valueChanged(TreeSelectionEvent evt) {
		TreePath path = evt.getPath();
		String selectedTerm = path.getLastPathComponent().toString();
		Employee emp = et.getChild(boss, selectedTerm);
		if (emp != null)
			cost.setText(new Float(emp.getSalary()).toString());
	}
}
