package decorator.jcooper;

import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JComponent;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class ButtonDecoratorTeam {
	precedence CoolDecorator, SlashDecorator;

	public abstract class Decorator {
		public abstract void paint(Graphics g);
	}

	public class CoolDecorator implements ILowerable, Decorator playedBy MyButton
	base when (ButtonDecoratorTeam.this.hasRole(base, CoolDecorator.class))
	{	
		private static final long serialVersionUID = 0L;
		boolean mouse_over;    //true when mouse over button

		public CoolDecorator(MyButton c) {
			mouse_over = false;
			//catch mouse movements in base class
			((MyButton)this.lower()).addMouseListener(
				new MouseAdapter() {
					public void mouseEntered(MouseEvent e) {
						mouse_over=true;     //set flag when mouse over
						repaintBase();
					}
					public void mouseExited(MouseEvent e) {
						mouse_over=false;    //clear flag when mouse not over
						repaintBase();
					}
				});
		}
		public void repaintBase(){
			((MyButton)this.lower()).repaint();
		}
		/* paint the button */
		public void paint(Graphics g) {
			//if the mouse is not over the button
			//erase the borders
			if(! mouse_over) {
				Dimension size = getComponentSize((MyButton)this.lower());
				g.setColor(Color.lightGray);
				g.drawRect(0, 0, size.width-1, size.height-1);
				g.drawLine(size.width-2, 0, size.width-2, size.height-1);
				g.drawLine(0, size.height-2, size.width-2, size.height-2);
			}
		}
		paint <- after paint;
	}

	public class SlashDecorator implements ILowerable, Decorator playedBy MyButton
	base when (ButtonDecoratorTeam.this.hasRole(base, SlashDecorator.class))
	{		
		public void paint(Graphics g) {
			Dimension size = getComponentSize((MyButton)this.lower());
			g.setColor(Color.red);
			g.drawLine(0, 0, size.width,size.height);
		}
		paint <- after paint;
	}

	public MyButton addSlashDecorator(MyButton c){
		new SlashDecorator(c);
		return c;
	}
	public MyButton addCoolDecorator(MyButton c){
		new CoolDecorator(c);
		return c;
	}
	private Dimension getComponentSize(JComponent c){
		return new Dimension(c.getWidth(),c.getHeight());
	}
	
}
