package facade.hk;

/**
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class FacadeTeam {

	protected class StringTransformerRole playedBy StringTransformer {
		public abstract static String transformToUpper(String s);
		transformToUpper -> transformToUpper;

		public abstract static String transformToLower(String s);
		transformToLower -> transformToLower;
	}

	protected class RegularScreenRole playedBy RegularScreen {
		public abstract static void print(String s);
		print -> print;

		public abstract static void newline();
		newline -> newline;
	}

	protected class DecorationRole playedBy Decoration {
		public abstract static String getDecoration();
		getDecoration -> getDecoration;
	}

	/**
	 * Prints a string using RegularScreenRole.
	 * 
	 * @param s   the string to print
	 */
	public void printNormal(String s) {
		RegularScreenRole.print(s);
		RegularScreenRole.newline();
	}

	/**
	 * Prints a two versions of string with decorations using
	 * RegularScreen and Decoration.
	 * 
	 * @param s   the string to print
	 */
	public void printFancy(String s) {
		printDecoration();
		RegularScreenRole.print(StringTransformerRole.transformToUpper(s
				+ " (uppercase)"));
		RegularScreenRole.newline();
		printDecoration();
		RegularScreenRole.print(StringTransformerRole.transformToLower(s
				+ " (lowercase)"));
		RegularScreenRole.newline();
		printDecoration();
	}

	/**
	 * Prints a decorator string.
	 */
	public void printDecoration() {
		RegularScreenRole.print(DecorationRole.getDecoration());
		RegularScreenRole.newline();
		RegularScreenRole.newline();
	}
}
