package facade.jcooper;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Vector;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 */
public class DatabaseFacade {
	private Connection _connection;
	private ResultSetFacade _resultSet;
	private DatabaseMetaData _dbMetaData;
	private String _catalog;
	private String _types[];
	private String _dbURL;

	public DatabaseFacade(String driver) {
		//Lets consider how we connect to a database.
		_types = new String[1];
		_types[0] = "TABLES"; // initialize type array
		//We first must load the database driver:
		try {
			Class.forName(driver);
		} // load the Bridge driver
		catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}

	public void close() {
		try {
			_connection.close();
		} catch (Exception e) {
			System.err.println("close error");
		}
	}

	public void open(String url, String cat) {
		_catalog = cat;
		_dbURL = url;
		try {
			_connection = DriverManager.getConnection(url);
			_dbMetaData = _connection.getMetaData(); // get the meta data
		} catch(SQLException sqle) {
			System.err.println(sqle.getMessage());
		}
		catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}

	public void reOpen() {
		try {
			_connection = DriverManager.getConnection(_dbURL);
			_dbMetaData = _connection.getMetaData(); // get the meta data
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}

	public String[] getTableNames() {
		String[] tbnames = null;
		Vector<String> tname = new Vector<String>();
		// add the table names to a Vector
		// since we don't know how many there are
		try {
			_resultSet = new ResultSetFacade(
				_dbMetaData.getTables(_catalog, null, "%", _types)
			);
		} catch (Exception e) {
			System.err.println("Exception thrown in " + getClass().getName() + ".getTableNames:");
			e.printStackTrace();
		}
		if(_resultSet == null) //MPM
			return null;
		while (_resultSet.hasMoreElements())
			tname.addElement(_resultSet.getColumnValue("TABLE_NAME"));

		// copy the table names into a String array
		tbnames = new String[tname.size()];
		for (int i = 0; i < tname.size(); i++)
			tbnames[i] = (String) tname.elementAt(i);
		return tbnames;
	}

	public String[] getTableMetaData() {
		// return the table type information
		_resultSet = null;
		try {
			_resultSet = new ResultSetFacade(_dbMetaData.getTables(_catalog, null, "%", _types));
		} catch (Exception e) {
			System.err.println("Exception thrown in getTableMetaData:\n" + e.getMessage());
		}
		return _resultSet.getMetaData();
	}

	public String[] getColumnMetaData(String tablename) {
		// return the data on a column
		_resultSet = null;
		try {
			_resultSet = new ResultSetFacade(_dbMetaData.getColumns(_catalog, null, tablename,
					null));
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
		return _resultSet.getMetaData();
	}

	public String[] getColumnNames(String table) {
		// return an array of Column names
		String[] tbnames = null;
		Vector<String> tname = new Vector<String>();

		try {
			_resultSet = new ResultSetFacade(_dbMetaData.getColumns(_catalog, null, table, null));
			while (_resultSet.hasMoreElements())
				tname.addElement(_resultSet.getColumnValue("COLUMN_NAME"));
		} catch (Exception e) {
			System.err.println(e);
		}

		tbnames = new String[tname.size()];
		for (int i = 0; i < tname.size(); i++)
			tbnames[i] = (String) tname.elementAt(i);
		return tbnames;
	}

	public String getColumnValue(String table, String columnName) {
		// return the value of a given column
		String res = null;
		try {
			if (table.length() > 0)
				_resultSet = execute("Select " + columnName + " from " + table
						+ " order by " + columnName);
			if (_resultSet.hasMoreElements())
				res = _resultSet.getColumnValue(columnName);
		} catch (Exception e) {
			System.err.println("Column value error" + columnName
					+ e.getMessage());
		}
		return res;
	}

	public String getNextValue(String columnName) {
		// return the next value in that column
		// using the remembered resultSet
		String res = "";
		try {
			if (_resultSet.hasMoreElements())
				res = _resultSet.getColumnValue(columnName);
		} catch (Exception e) {
			System.err.println("next value error" + columnName + e.getMessage());
		}
		return res;
	}

	public ResultSetFacade execute(String sql) {
		// execute an SQL query on this database
		_resultSet = null;
		try {
			Statement stmt = _connection.createStatement();
			_resultSet = new ResultSetFacade(stmt.executeQuery(sql));
		} catch (Exception e) {
			System.err.println("execute error: " + e.getMessage());
		}
		return _resultSet;
	}
}
