package facade.jcooper;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.util.Vector;

/**
 * This class is a higher level abstraction of the JDBC ResultSet object
 * 
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 */
public class ResultSetFacade {
	private ResultSet _resultSet;
	private ResultSetMetaData _resultSetMetadata;
	private int _numCols;

	public ResultSetFacade(ResultSet resultSet) {
		_resultSet = resultSet;
		try {
			// get the meta data and column count at once
			_resultSetMetadata = _resultSet.getMetaData();
			_numCols = _resultSetMetadata.getColumnCount();
		} catch (Exception e) {
			System.err.println("resultset error" + e.getMessage());
		}
	}

	public String[] getMetaData() {
		// returns an array of all the column names
		// or other meta data
		String md[] = new String[_numCols];
		try {
			for (int i = 1; i <= _numCols; i++)
				md[i - 1] = _resultSetMetadata.getColumnName(i);
		} catch (Exception e) {
			System.err.println("meta data error" + e.getMessage());
		}
		return md;
	}

	public boolean hasMoreElements() {
		try {
			return _resultSet.next();
		} catch (Exception e) {
			System.err.println("next error " + e.getMessage());
			return false;
		}
	}

	public String[] nextElement() {
		// copies contents of row into string array
		String[] row = new String[_numCols];
		try {
			for (int i = 1; i <= _numCols; i++)
				row[i - 1] = _resultSet.getString(i);
		} catch (Exception e) {
			System.err.println("next element error" + e.getMessage());
		}
		return row;
	}

	public String getColumnValue(String columnName) {
		String res = "";
		try {
			res = _resultSet.getString(columnName);
		} catch (Exception e) {
			System.err.println("Column value error:" + columnName
					+ e.getMessage());
		}
		return res;
	}

	public String getColumnValue(int i) {
		String res = "";
		try {
			res = _resultSet.getString(i);
		} catch (Exception e) {
			System.err.println("Column value error: " + i + " "
					+ e.getMessage());
		}
		return res;
	}

	public void finalize() {
		try {
			_resultSet.close();
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
	public Vector<String[]> makeTables() {
		Vector<String[]> tables = new Vector<String[]>();
		String t[] = getMetaData();
		tables.addElement(t);
		while (hasMoreElements())
			tables.addElement(nextElement());
		return tables;
	}
}
