/*
   30 de Jul de 2010
 */
package facade.jcooper;

import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.List;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.util.Vector;

public team class Window4FacadeTeam {

	protected class DBDisplay extends Frame implements ActionListener,
			ItemListener playedBy DatabaseFacade {
		private static final long serialVersionUID = 0L;

		private List Tables, Columns, Data;
		private TextArea query;
		private Button Search, Quit;

		public DBDisplay(DatabaseFacade db) {
			super("Database demonstration");
			setGUI();
			open("jdbc:odbc:groceries.mdb", null);
			System.out.println("creating table names...");
			String tnames[] = getTableNames();
			System.out.println("tnames is " + tnames);

			loadList(Tables, tnames);
			String queryText = "SELECT DISTINCTROW FoodName, StoreName, Price "
					+ "FROM (Food INNER JOIN FoodPrice ON Food.FoodKey = FoodPrice.FoodKey) "
					+ "INNER JOIN Stores ON FoodPrice.StoreKey = Stores.StoreKey "
					+ "WHERE (((Food.FoodName)=\'Oranges\')) ORDER BY FoodPrice.Price;";

			// System.out.println("queryText:\n" + queryText);
			query.setText(queryText);
		}

		public abstract void open(String url, String cat);
		open -> open;

		public abstract String[] getTableNames();
		getTableNames -> getTableNames;

		public abstract ResultSetFacade execute(String sql);
		execute -> execute;

		public abstract String[] getColumnNames(String table);
		getColumnNames -> getColumnNames;

		public abstract String getColumnValue(String table, String columnName);
		getColumnValue -> getColumnValue;

		public abstract String getNextValue(String columnName);
		getNextValue -> getNextValue;

		private void setGUI() {
			setBackground(Color.lightGray);
			setLayout(new BorderLayout());
			Panel pn = new Panel();
			add("North", pn);
			pn.setLayout(new GridLayout(1, 3));
			pn.add(new Label("Tables"));
			pn.add(new Label("Columns"));
			pn.add(new Label("Data"));
			Panel pc = new Panel();
			add("Center", pc);
			pc.setLayout(new GridLayout(1, 3));
			pc.add(Tables = new List(15));
			pc.add(Columns = new List(15));
			pc.add(Data = new List(15));
			Tables.addItemListener(this);
			Columns.addItemListener(this);
			Panel ps = new Panel();
			add("South", ps);
			ps.add(query = new TextArea("", 3, 40));
			addPanel(ps, Search = new Button("Run Query"));
			addPanel(ps, Quit = new Button("Quit"));
			Search.addActionListener(this);
			Quit.addActionListener(this);
			setBounds(100, 100, 500, 300);
			setVisible(true);
		}

		private void addPanel(Panel ps, Component c) {
			Panel p = new Panel();
			ps.add(p);
			p.add(c);
		}

		private void loadList(List list, String[] s) {
			list.removeAll();
			for (int i = 0; i < s.length; i++)
				list.add(s[i]);
		}

		public void actionPerformed(ActionEvent e) {
			Object obj = e.getSource();
			if (obj == Quit)
				System.exit(0);
			if (obj == Search)
				clickedSearch();
		}

		public void itemStateChanged(ItemEvent e) {
			Object obj = e.getSource();
			if (obj == Tables)
				showColumns();
			if (obj == Columns)
				showData();
		}

		private void showColumns() {
			String cnames[] = getColumnNames(Tables.getSelectedItem());
			loadList(Columns, cnames);
		}

		private void showData() {
			String colname = Columns.getSelectedItem();
			String colval = getColumnValue(Tables.getSelectedItem(), colname);
			Data.setVisible(false);
			Data.removeAll();
			Data.setVisible(true);
			colval = getNextValue(Columns.getSelectedItem());
			while (colval.length() > 0) {
				Data.add(colval);
				colval = getNextValue(Columns.getSelectedItem());
			}
		}

		private void clickedSearch() {
			ResultSetFacade rs = execute(query.getText());
			/* String cnames[] = */rs.getMetaData();
			Columns.removeAll();
			QueryDialog q = new QueryDialog(this, rs);
			q.show();
		}
	}

	protected class TextPanel extends Panel {
		private static final long serialVersionUID = 0L;
		private Vector<String[]> _tables;

		public TextPanel(Vector<String[]> tables) {
			_tables = tables;
		}
		static final long serialVersionUID = 0L;

		public void paint(Graphics g) {
			String s[];
			int x = 0;
			int y = g.getFontMetrics().getHeight();
			int deltaX = (int) 1.5f
					* (g.getFontMetrics().stringWidth("wwwwwwwwwwwwww"));
			for (int i = 0; i < _tables.size(); i++) {
				s = (String[]) _tables.elementAt(i);
				for (int j = 0; j < s.length; j++) {
					String st = s[j];
					g.drawString(st, x, y);
					x += deltaX;
				}
				x = 0;
				y += g.getFontMetrics().getHeight();
				if (i == 0)
					y += g.getFontMetrics().getHeight();
			}
		}
	}

	protected class QueryDialog extends Dialog implements ActionListener {
		private static final long serialVersionUID = 0L;

		private ResultSetFacade _resultSet;
		private Button _OKButton;
		private TextPanel _textPanel;
		private Vector<String[]> _tables;

		public QueryDialog(Frame f, ResultSetFacade r) {
			super(f, "Query Result");
			_resultSet = r;
			setLayout(new BorderLayout());
			_OKButton = new Button("OK");
			Panel p = new Panel();
			add("South", p);
			p.add(_OKButton);
			_OKButton.addActionListener(this);
			_textPanel = new TextPanel(_tables);
			_textPanel.setBackground(Color.white);
			add("Center", _textPanel);
			makeTables();
			setBounds(100, 100, 500, 300);
			setVisible(true);
			repaint();
		}

		private void makeTables() {
			_tables = new Vector<String[]>();
			String t[] = _resultSet.getMetaData();
			_tables.addElement(t);
			while (_resultSet.hasMoreElements()) {
				_tables.addElement(_resultSet.nextElement());
			}
		}

		public void actionPerformed(ActionEvent e) {
			setVisible(false);
		}
	}

	public static void loadList(List list, String[] s) {
		list.removeAll();
		for (int i = 0; i < s.length; i++)
			list.add(s[i]);
	}

	public Window4FacadeTeam(DatabaseFacade as DBDisplay db) {
		// Without this, the callin bindings have no effect.
		activate(ALL_THREADS);
	}
}
