package factoryMethod.hk;

import java.awt.Point;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class GUIComponentCreatorTeam {

	public abstract class Product {
//	    public Product(){
//	    }

	   	 /**
         * The factory method to be concretized by roles overriding Product role.
         * @returns the created ConcreteProduct
         */
    	public abstract JComponent newProduct();
	    	
	    /**
	     * Another factory method to create a title for the GUI frame created
	     * by this class.
	     * @returns the title for the GUI frame
	     */
	    public abstract String getTitle(); 
	
	    /** 
	     * the position for the next frame to be created (on the screen). This
	     * variable is used to make sure new frames appear staggered and don't 
	     * entirely overlap with existing frames.
	     */
	   private Point lastFrameLocation = new Point(0, 0);

	    /** 
	     * Creates a JFrame, puts the JComponent that
	     * is created by the factory method into it and displays the frame. This
	     * Method also provides a WindowListener. 
	     */
	    public final void showFrame() {
	        JFrame frame = new JFrame(getTitle());
	   		frame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {System.exit(0);}
			}); 
			JPanel panel = new JPanel();
			panel.add(newProduct());
			
			frame.getContentPane().add(panel);
			frame.pack();    
			frame.setLocation(lastFrameLocation);
			lastFrameLocation.translate(75, 75);
			frame.setVisible(true);  
	    }
    }
	public abstract Product createProduct();

	final public static void createFrame(final GUIComponentCreatorTeam fam, Product<@fam> c) {
		c.showFrame();
	}
}		
