package factoryMethod.hk;

/**
 * Implements the driver for the FactoryMethod design pattern example.
 *
 * Intent: Define an interface for creating an object, but let subclasses 
 * decide which class to instantiate. Factory Method lets a class defer 
 * instantiation to subclasses.
 *
 * Participating objects are ButtonCreatorTeam and LabelCreatorTeam as ConcreteCreator's. 
 *
 * In this example, the factory method createProduct creates a JComponent (a button and
 * a label, respectively). The method createFrame() uses the factory method to show a
 * little GUI, with either the button or the label.
 * In one case, the created frame contains a button, in the other a simple label.
 *
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {

    /**
     * Implements the driver for the FactoryMethod design pattern example. 
     *
     * In this example, the factory method createProduct
     * creates a JComponent (a button and a label, respectively). The method createFrame() 
     * uses the factory methods to show a little GUI. In one case, the created frame
     * contains a button, in the other a simple label.
     */
    public static void main(String[] args) {
        final GUIComponentCreatorTeam buttonFac = new ButtonCreatorTeam();
        final GUIComponentCreatorTeam labelFac = new LabelCreatorTeam();

        Product<@buttonFac> button = buttonFac.createProduct();
        Product<@labelFac> label = labelFac.createProduct();

        GUIComponentCreatorTeam.createFrame(buttonFac, button);
        GUIComponentCreatorTeam.createFrame(labelFac, label);

//      Family polymorphism prevents members from different families to mix: 
//      GUIComponentCreatorTeam.createFrame(buttonFac, label);
//      GUIComponentCreatorTeam.createFrame(labelFac, button);
    }
}
