/*
   28 de Jun de 2010
*/
package factoryMethod.jcooper;

/**
 * This example uses the 'Transparent Role' idiom.
 * Roles from this team do not use playedBy.
 * 
 * @author Miguel P. Monteiro
 */
public team class NamerFactoryTeam {
	protected class Namer implements INamer {
		protected String last; // store last name here
		protected String first; // store first name here

		public String getFirst() {
			return first; // return first name
		}

		public String getLast() {
			return last; // return last name
		}
	}

	protected class FirstFirst extends Namer { // split first last
		public FirstFirst(String s) {
			int i = s.lastIndexOf(" "); // find sep space
			if (i > 0) {
				// left is first name
				first = s.substring(0, i).trim();
				// right is last name
				last = s.substring(i + 1).trim();
			} else {
				first = ""; // put all in last name
				last = s; // if no space
			}
		}
	}

	protected class LastFirst extends Namer { // split last, first
		public LastFirst(String s) {
			int i = s.indexOf(","); // find comma
			if (i > 0) {
				// left is last name
				last = s.substring(0, i).trim();
				// right is first name
				first = s.substring(i + 1).trim();
			} else {
				last = s; // put all in last name
				first = ""; // if no comma
			}
		}
	}

	/*
	 * This is the factory method. It still exists in the example, despite family
	 * polymorphism, because choice of concrete class to instantiate is made
	 * dynamically and depending on a value obtained from the end user. 
	 */
	public INamer getNamer(String entry) {
		int i = entry.indexOf(","); // comma determines name order
		if (i > 0)
			return new LastFirst(entry); // return one class
		else
			return new FirstFirst(entry); // or the other
	}
}
