/*
   24 de Jun de 2010
*/
package factoryMethod.jcooper;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import jcooper_support.JxFrame;
/**
 * This class emulates the GUI for Factory Method shown in Cooper's book.
 * Sources for the GUI are not shown in the book. This class recreates the GUI
 * on the basis of a figure.
 * Creational patterns from Cooper's book use AWT rather than swing. However,
 * for this recreation a swing based implementation is used.
 * 
 * @author Miguel P. Monteiro
 */
public class Window4FactoryMethod extends JxFrame {
	private static final long serialVersionUID = 0;
	private JTextField _fullNameField;
	private JTextField _firstNameField;
	private JTextField _lastNameField;
	private NamerFactoryTeam _factory;

	public Window4FactoryMethod(NamerFactoryTeam factory) {
		super("Name Divider");
		_factory = factory;
		getContentPane().setLayout(new BorderLayout());
		JPanel jp = new JPanel();
		getContentPane().add(jp);
		JLabel label = new JLabel("Enter name:");
		label.setForeground(Color.BLUE);
		jp.add(label);
		_fullNameField = new JTextField(35); 
		jp.add(_fullNameField);
		JLabel firstNameLabel = new JLabel("First name:");
		jp.add(firstNameLabel);
		_firstNameField = new JTextField(25); 
		jp.add(_firstNameField);
		JLabel lastNameLabel = new JLabel("Last name:");
		jp.add(lastNameLabel);
		_lastNameField = new JTextField(25); 
		jp.add(_lastNameField);

		JButton computeButton = new JButton("Compute");
		computeButton.addActionListener(new ComputeAction());
		JButton clearButton = new JButton("Clear");
		clearButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent dummy) {
				_fullNameField.setText("");
				_firstNameField.setText("");
				_lastNameField.setText("");
			}
		});
		JButton quitButton = new JButton("Quit");
		quitButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent dummy) {
				System.exit(0);
			}
		});
		jp.add(computeButton);
		jp.add(clearButton);
		jp.add(quitButton);
		setSize(new Dimension(300,200));
		setVisible(true);
	}

	private class ComputeAction implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			String inputName = _fullNameField.getText();
			INamer namer = _factory.getNamer(inputName);
			_firstNameField.setText(namer.getFirst());
			_lastNameField.setText(namer.getLast());
		}
	}
}
