package flyweight.hk;

/**
 * Implements the driver for the Flyweight design pattern example.
 * 
 * Intent: Use sharing to support large numbers of fine-grained objects
 * efficiently.
 * 
 * Participating Flyweight classes are CharacterFlyweight and
 * WhitespaceFlyweight.
 * 
 * Intrinsic state: The character to print, Extrinsic state: Whether the char is
 * upper case or lower case
 * 
 * This example creates a sentence out of PrintableFlyweights (characters and
 * whitespaces).
 * 
 * This implementation is analoguous to the GoF book's description. For this
 * version, we only use two classes for the flyweights, CharacterFlyweight and
 * WhitespaceFlyweight. Plus, this implementation uses an optimized factory that
 * keeps a hashtable of created Flyweights and employs creation of demand.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {

	/**
	 * Implements the driver for the Flyweight design pattern example.
	 * 
	 * This example creates a sentence out of PrintableFlyweights (characters
	 * and whitespaces).
	 */
	public static void main(String[] args) {
		System.out.println("This is a test for the java Flyweight pattern implementation.");
		System.out.println("The client will use char flyweights to print the phrase");
		System.out.println("\"This Is A Test\".\n");
		System.out.println("Testing Pattern: Flyweight - STARTING\n");

		FlyweightTeam pff = new FlyweightTeam();

		PrintableFlyweight T = pff.getFlyweight('t');
		PrintableFlyweight H = pff.getFlyweight('h');
		PrintableFlyweight I = pff.getFlyweight('i');
		PrintableFlyweight S = pff.getFlyweight('s');
		PrintableFlyweight A = pff.getFlyweight('a');
		PrintableFlyweight E = pff.getFlyweight('e');
		PrintableFlyweight Empty = pff.getFlyweight(' ');

		// Printing: "This Is A Test"

		T.print(true);
		H.print(false);
		I.print(false);
		S.print(false);

		Empty.print(true);

		I.print(true);
		S.print(false);

		Empty.print(true);

		A.print(true);

		Empty.print(true);

		T.print(true);
		E.print(false);
		S.print(false);
		T.print(false);

		System.out.println();

		System.out.println("\nTesting Pattern: State - FINISHED");
	}
}