package flyweight.jcooper;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.Vector;

import javax.swing.JPanel;

import jcooper_support.JxFrame;


/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Miguel P. Monteiro - refactored the Java version forming the basis for this example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Window4Flyweight extends JxFrame implements MouseMotionListener {
	private static final long serialVersionUID = 0L;

	private Vector<String> _names;
	private FlyweightTeam _folderFactoryTeam;
	private final int Top = 30, Left = 30;
	private final int W = 50, H = 30;
	private final int VSpace = 80, HSpace = 70, HCount = 3;
	private String _selectedName = "";

	public Window4Flyweight(FlyweightTeam folderFactoryTeam) {
		super("Flyweight Canvas");
		_folderFactoryTeam = folderFactoryTeam;
		loadNames();
		JPanel jp = new JPanel();
		getContentPane().add(jp);
		setSize(new Dimension(300, 300));
		addMouseMotionListener(this);
		setVisible(true);
		repaint();
	}

	private void loadNames() {
		_names = new Vector<String>();
		_names.addElement("Alan");
		_names.addElement("Barry");
		_names.addElement("Charlie");
		_names.addElement("Dave");
		_names.addElement("Edward");
		_names.addElement("Fred");
		_names.addElement("George");
		_selectedName = "Charlie";
	}

	public void paint(Graphics g) {
		Folder folder;
		String name;
		int j = 0; // count number in row
		int row = Top; // start in upper left
		int x = Left;
		// go through all the names and folders
		for (int i = 0; i < _names.size(); i++) {
			name = (String) _names.elementAt(i);
			if (name.equals(_selectedName))
				folder = (Folder) _folderFactoryTeam.getFlyweight(true);
			else
				folder = (Folder) _folderFactoryTeam.getFlyweight(false);
			// have that folder draw itself at this spot
			folder.Draw(g, x, row, name);
			x = x + HSpace; // change to next posn
			j++;
			if (j >= HCount) // reset for next row
			{
				j = 0;
				row += VSpace;
				x = Left;
			}
		}
	}

	public void mouseMoved(MouseEvent e) {
		int j = 0; // count number in row
		int row = Top; // start in upper left
		int x = Left;
		// go through all the names and folders
		for (int i = 0; i < _names.size(); i++) {
			// see if this folder contains the mouse
			Rectangle r = new Rectangle(x, row, W, H);
			if (r.contains(e.getX(), e.getY())) {
				_selectedName = (String) _names.elementAt(i);
				repaint();
			}
			x = x + HSpace; // change to next posn
			j++;
			if (j >= HCount) // reset for next row
			{
				j = 0;
				row += VSpace;
				x = Left;
			}
		}
	}

	public void mouseDragged(MouseEvent e) {
	}
}
