package interpreter.hk;

/**
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class InterpreterTeam extends InterpreterAbstTeam {

	public class NotExpression extends BooleanExpression {
		/**
		 * The Expression this Expression negates
		 */
		protected BooleanExpression exp = null;

		/**
		 * Creates a new NOT Expression negating the argument
		 * Expression
		 * 
		 * @param exp the Expression to negate
		 */
		public NotExpression(BooleanExpression exp) {
			this.exp = exp;
		}

		/**
		 * Evaluates this Expression
		 * 
		 * @return the boolean value of the Expression
		 */
		public boolean evaluate() {
			return (!exp.evaluate());
		}

		/**
		 * Replaces a variable with an Expression
		 * 
		 * @param name
		 *            the name of the variable
		 * @param exp
		 *            the Expression to replace the variable
		 * @return a copy of this Expression with the variable replaced
		 */
		public BooleanExpression replace(String name, BooleanExpression exp) {
			return new NotExpression(this.exp.replace(name, exp));
		}

		/**
		 * Copies this Expression
		 * 
		 * @returns the copied Expression
		 */
		public BooleanExpression copy() {
			return new NotExpression(exp.copy());
		}
	}

	public class AndExpression extends BooleanExpression {
		/**
		 * stores the first part of this AND Expression
		 */
		protected BooleanExpression expression1;

		/**
		 * stores the second part of this AND Expression
		 */
		protected BooleanExpression expression2;

		/**
		 * Creates a new AND Expression with the given parts
		 * 
		 * @param expression1
		 *            the first Expression
		 * @param expression1
		 *            the second Expression
		 */
		public AndExpression(BooleanExpression op1, BooleanExpression op2) {
			this.expression1 = op1;
			this.expression2 = op2;
		}

		/**
		 * Evaluates this Expression in the given
		 * <code>VariableContext</code>
		 * 
		 * @param c
		 *            the Context to evaluate the Expression in
		 * @return the boolean value of this AND Expression
		 */
		public boolean evaluate() {
			return (expression1.evaluate() && expression2.evaluate());
		}

		/**
		 * Replaces a variable with an Expression
		 * 
		 * @param name
		 *            the name of the variable
		 * @param exp
		 *            the Expression to replace the variable
		 * @return a copy of this Expression with the variable replaced
		 */
		public BooleanExpression replace(String name, BooleanExpression exp) {
			return new AndExpression(expression1.replace(name, exp),
					expression2.replace(name, exp));
		}

		/**
		 * Copies this Expression
		 * 
		 * @returns the copied Expression
		 */
		public BooleanExpression copy() {
			return new AndExpression(expression1.copy(), expression2.copy());
		}
	}

	public class OrExpression extends BooleanExpression {
		/**
		 * stores the first part of this OR Expression
		 */

		protected BooleanExpression expression1;

		/**
		 * stores the second part of this OR Expression
		 */
		protected BooleanExpression expression2;

		/**
		 * Creates a new OR Expression with the given parts
		 * 
		 * @param expression1
		 *            the first Expression
		 * @param expression1
		 *            the second Expression
		 */
		public OrExpression(BooleanExpression op1, BooleanExpression op2) {
			this.expression1 = op1;
			this.expression2 = op2;
		}

		/**
		 * Evaluates this Expression in the given Context
		 * 
		 * @param c
		 *            the context to evaluate the Expression in
		 * @returns the boolean value of the Expression
		 */
		public boolean evaluate() {
			return (expression1.evaluate() || expression2.evaluate());
		}

		/**
		 * Replaces a variable with an Expression
		 * 
		 * @param name
		 *            the name of the variable
		 * @param exp
		 *            the Expression to replace the variable
		 * @returns a copy of this Expression with the variable replaced
		 */
		public BooleanExpression replace(String name, BooleanExpression exp) {
			return new OrExpression(expression1.replace(name, exp), expression2
					.replace(name, exp));
		}

		/**
		 * Copies this Expression
		 * 
		 * @returns the copied Expression
		 */
		public BooleanExpression copy() {
			return new OrExpression(expression1.copy(), expression2.copy());
		}
	}

}
