package interpreter.jcooper;


import java.util.StringTokenizer;
import java.util.Vector;

import jcooper_support.ICommand;
import jcooper_support.JawtList;
import jcooper_support.Kid;
import jcooper_support.KidData;
import jcooper_support.Stack;

public team class ParserTeam {

	protected class ParseObject {
		public static final int VERB = 1000, VAR = 1010, MULTVAR = 1020;

		protected int value;
		protected int type;

		public int getValue() {
			return value;
		}
		public int getType() {
			return type;
		}
	}

	protected class ParseVar extends ParseObject {
		static final public int FRNAME = 0, LNAME = 1, AGE = 2, CLUB = 3,
				TIME = 4, tabMAX = 5;

		public ParseVar(String s) {
			s = s.toLowerCase();
			value = -1;
			type = VAR;
			if (s.equals("frname"))
				value = FRNAME;
			if (s.equals("lname"))
				value = LNAME;
			if (s.equals("age"))
				value = AGE;
			if (s.equals("club"))
				value = CLUB;
			if (s.equals("time"))
				value = TIME;
		}
		public boolean isLegal() {
			return (value >= 0);
		}
	}
	protected class MultVar extends ParseVar {
		private Vector<ParseObject> _multVector;

		public MultVar(ParseObject v1, ParseObject v2) {
			super("");
			_multVector = new Vector<ParseObject>();
			_multVector.addElement(v1);
			_multVector.addElement(v2);
			type = MULTVAR;
		}

		public MultVar() {
			super("");
			_multVector = new Vector<ParseObject>();
			type = MULTVAR;
		}

		public void add(ParseObject v1) {
			_multVector.addElement(v1);
		}

		public Vector<ParseObject> getVector() {
			return _multVector;
		}
	}

	protected class ParseVerb extends ParseObject {
		public static final int PRINT = 100, SORTBY = 110, THENBY = 120;
		protected Vector<ParseObject> args;

		public ParseVerb(String s) {
			args = new Vector<ParseObject>();
			s = s.toLowerCase();
			value = -1;
			type = VERB;
			if (s.equals("print"))
				value = PRINT;
			if (s.equals("sortby"))
				value = SORTBY;
		}

		public ParseVerb getVerb(String s) {
			switch (value) {
			case PRINT:
				return new Print(s);
			case SORTBY:
				return new Sort(s);
			}
			return null;
		}

		public void addArgs(MultVar mv) {
			args = mv.getVector();
		}

		public void addArgs(ParseObject p) {
			args.addElement(p);
		}

		public boolean isLegal() {
			return (value >= 0);
		}
	}

	protected class Verb extends ParseVerb implements ICommand {
		protected KidData data;
		protected JawtList ptable;

		public Verb(String s) {
			super(s);
		}

		public void execute() {
		}

		public void setData(KidData dt, JawtList pt) {
			data = dt;
			ptable = pt;
		}
	}

	protected class Sort extends Verb {
		private Kid[] _kids;

		public Sort(String s) {
			super(s);
			value = SORTBY;
		}

		public void execute() {
			int sortKey;

			_kids = data.getData();
			for (int a = 0; a < args.size(); a++) {
				ParseVar v = (ParseVar) args.elementAt(a);
				if (v instanceof MultVar) {
					MultVar mv = (MultVar) v;
					Vector<ParseObject> mvec = mv.getVector();
					for (int k = mvec.size() - 1; k >= 0; k--) {
						ParseVar pv = (ParseVar) mvec.elementAt(k);
						sortKey = pv.getValue();
						sortByKey(sortKey);
					}
				} else {
					sortKey = v.getValue();
					sortByKey(sortKey);
				}
			}
		}

		private void sortByKey(int sortkey) {
			for (int i = 0; i < _kids.length; i++)
				for (int j = i + 1; j < _kids.length; j++)
					if (compare(i, j, sortkey)) {
						Kid tmp = _kids[i];
						_kids[i] = _kids[j];
						_kids[j] = tmp;
					}
		}

		private boolean compare(int i, int j, int key) {
			boolean cval;

			switch (key) {
			case ParseVar.FRNAME:
				cval = _kids[i].getFrname().compareTo(_kids[j].getFrname()) > 0;
				break;
			case ParseVar.LNAME:
				cval = _kids[i].getLname().compareTo(_kids[j].getLname()) > 0;
				break;
			case ParseVar.CLUB:
				cval = _kids[i].getClub().compareTo(_kids[j].getClub()) > 0;
				break;
			case ParseVar.AGE:
				cval = _kids[i].getAge() > _kids[j].getAge();
				break;
			case ParseVar.TIME:
				cval = _kids[i].getTime() > _kids[j].getTime();
				break;
			default:
				cval = false;
			}
			return cval;
		}
	}

	protected class Print extends Verb {
		private Kid[] _kids;

		public Print(String s) {
			super(s);
			value = PRINT;
		}

		public void execute() {
			String pline;

			_kids = data.getData();
			// ptable.setDims(kids.length, args.size());
			ptable.clear();

			for (int i = 0; i < _kids.length; i++) {
				pline = ""; // line in output list
				for (int j = 0; j < args.size(); j++) {

					ParseVar v = (ParseVar) args.elementAt(j);
					if (v instanceof MultVar) {
						MultVar mv = (MultVar) v;
						Vector<ParseObject> vlist = mv.getVector();
						for (int k = 0; k < vlist.size(); k++) {
							ParseVar pv = (ParseVar) vlist.elementAt(k);
							// System.out.print(kids[i].getData(pv.getValue())+"   ");
							pline += _kids[i].getData(pv.getValue()) + "   ";
						}
					} else
						pline += _kids[i].getData(v.getValue()) + "   ";
				}
				ptable.add(pline);
			}
			ptable.validate();
			ptable.repaint();
		}
	}

	private Stack<ParseObject> _stack;
	private Vector<ParseObject> _actionList;
	private KidData _kidData;
	private JawtList _printTable; //	PrintTable ptable;

	public ParserTeam(String line) {
		_stack = new Stack<ParseObject>();
		_actionList = new Vector<ParseObject>();
		StringTokenizer tok = new StringTokenizer(line);
		while (tok.hasMoreElements()) {
			ParseObject token = tokenize(tok.nextToken());
			if (token != null)
				_stack.push(token);
		}
	}

	public void setData(KidData k, JawtList pt) {
		_kidData = new KidData(k.getData());
		_printTable = pt;
	}

	// executes parse of command line
	public void execute() {
		while (_stack.hasMoreElements()) {
			if (topStack(ParseObject.VAR, ParseObject.VAR)) {
				// reduce (Var Var) to Multvar
				ParseVar v = (ParseVar) _stack.pop();
				ParseVar v1 = (ParseVar) _stack.pop();
				MultVar mv = new MultVar(v1, v);
				_stack.push(mv);
			}
			// reduce MULTVAR VAR to MULTVAR
			if (topStack(ParseObject.MULTVAR, ParseObject.VAR)) {
				MultVar mv = new MultVar();
				MultVar mvo = (MultVar) _stack.pop();
				ParseVar v = (ParseVar) _stack.pop();
				mv.add(v);
				Vector<ParseObject> mvec = mvo.getVector();
				for (int i = 0; i < mvec.size(); i++)
					mv.add((ParseVar) mvec.elementAt(i));
				_stack.push(mv);
			}
			if (topStack(ParseObject.VAR, ParseObject.MULTVAR)) {
				// reduce (Multvar Var) to Multvar
				ParseVar v = (ParseVar) _stack.pop();
				MultVar mv = (MultVar) _stack.pop();
				mv.add(v);
				_stack.push(mv);
			}
			// reduce Verb Var to Verb containing vars
			if (topStack(ParseObject.VAR, ParseObject.VERB)) {
				addArgsToVerb();
			}
			// reduce Verb MultVar to Verb containing vars
			if (topStack(ParseObject.MULTVAR, ParseObject.VERB)) {
				addArgsToVerb();
			}
			// move top verb to action list
			if (_stack.top().getType() == ParseObject.VERB) {
				_actionList.addElement(_stack.pop());
			}
		}
		// now execute the verbs
		for (int i = 0; i < _actionList.size(); i++) {
			Verb v = (Verb) _actionList.elementAt(i);
			v.setData(_kidData, _printTable);
			v.execute();
		}
	}

	private void addArgsToVerb() {
		ParseObject v = _stack.pop();
		ParseVerb verb = (ParseVerb) _stack.pop();
		verb.addArgs(v);
		_stack.push(verb);
	}

	private boolean topStack(int c1, int c2) {
		return (_stack.top().getType() == c1) && (_stack.nextTop().getType() == c2);
	}

	private ParseObject tokenize(String s) {
		ParseObject obj = getVerb(s);
		if (obj == null)
			obj = getVar(s);
		return obj;
	}

	private ParseVerb getVerb(String s) {
		ParseVerb v;
		v = new ParseVerb(s);
		if (v.isLegal())
			return v.getVerb(s);
		return null;
	}

	private ParseVar getVar(String s) {
		ParseVar v;
		v = new ParseVar(s);
		if (v.isLegal())
			return v;
		return null;
	}
}
