/*
   23 de Out de 2010
*/
package iterator.hk;

import java.util.LinkedList;
//import java.util.Iterator;

/**
 * Implements a basic open list. This implementation is based on
 * java.util.LinkedList. In essence, this class acts as an adapter
 * for the Java class.
 * 
 * In this version, it is no longer necessary to implement the factory method
 * here to create the iterator.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Miguel P. Monteiro - adapted to Object Teams/Java
 */
public team class ListAggregateTeam implements SimpleList {
	private LinkedList<Integer> _list = new LinkedList<Integer>();

	protected class Iterator implements java.util.Iterator<Integer> {
		/**
		 * the positition of the current element
		 */
		protected int current;

		/**
		 * the list this iterator operates on
		 */
		protected LinkedList<Integer> list;

		/**
		 * Creates a new ReverseIterator from the given list.
		 * Family polymorphism guarantees that Iterator only collaborate
		 * with aggregates of this team.
		 * 
		 * @param list the list to generate an iterator from
		 */
		public Iterator() {
			this.list = ListAggregateTeam.this._list;
			current = this.list.size();
		}

		/**
		 * Returns true if the iteration has more elements.
		 * 
		 * @return true if the iteration has more elements
		 */
		public boolean hasNext() {
			return (current > 0);
		}

		/**
		 * This opional method is not implemented for this iterator.
		 */
		public void remove() {
			throw new UnsupportedOperationException("remove() not supported");
		}

		/**
		 * Returns the next element in the iteration.
		 * 
		 * @return the next element in the iteration.
		 */
		public Integer next() {
			if (!hasNext()) {
				throw new ArrayIndexOutOfBoundsException(
						"Iterator out of Bounds");
			} else {
				return list.get(--current);
			}
		}
	}

	/**
	 * Returns the number of elements in the list
	 * 
	 * @return the number of elements in the list
	 */
	public int count() {
		return _list.size();
	}

	/**
	 * Appends an object to the list. Since this is an open list, inserting
	 * elements is assumed to succeed.
	 * 
	 * @param integer
	 *            the object to append
	 * @return true if successful, false otherwise
	 */
	public boolean append(Integer integer) {
		_list.addLast(integer);
		return true;
	}

	/**
	 * Removes an object from the list
	 * 
	 * @param o
	 *            the object to remove
	 * @return true if successful, false otherwise
	 */
	public boolean remove(Integer integer) {
		return _list.remove(integer);
	}

	/**
	 * Returns an object from the list
	 * 
	 * @param index
	 *            the position of the object
	 * @return the object at position index
	 */
	public Integer get(int index) {
		return _list.get(index);
	}

	public java.util.Iterator<Integer> getReverseIterator() {
		return new Iterator();
	}
}
