package iterator.hk;

import java.util.Iterator;
/**
 * Implements the driver for the Iterator design pattern example. 
 *
 * Intent: Provide a way to access the elements of an aggregate object
 * sequentially without exposing its underlying representation.
 *
 * In this example, the concrete aggregate is a list that gets filled with 
 * five integer objects (1 to 5). The list is implemented by a team, which
 * returns an iterator that is used to print all elements in reverse order.
 *
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 * @author Miguel P. Monteiro - adapted from the previous HK2 version
 */
public class Main {
    /**
     * Prints all elements in the iterator to System.out.
     *
     * @param iter the iterator which elements are to be printed
     */
	private static void print(Iterator<Integer> iter) { 
		while(iter.hasNext())
			System.out.println(iter.next());
	}

    /**
     * Implements the driver for the Iterator design pattern example. 
     *
     * In this example, the concrete aggregate is a list that gets filled with 
     * five integer objects (1 to 5). The, the ReverseIterator is 
     * created and used to print all elements in reverse order.  
     *
     * @param args command line paramters, unused
     */
	public static void main(String[] args) {
		ListAggregateTeam listAggregate = new ListAggregateTeam();

		listAggregate.append(new Integer(1));
		listAggregate.append(new Integer(2));		
		listAggregate.append(new Integer(3));
		listAggregate.append(new Integer(4));
		listAggregate.append(new Integer(5)); 
		System.out.println("OT/J Example");
		System.out.println("List created, containing int objects 1, 2, 3, 4, 5.");

		Iterator<Integer> iter = listAggregate.getReverseIterator();

		System.out.println("Using ReverseIterator to print list elements in reverse order...");
		print(iter);

		System.out.print("done.");
	}
}
