package iterator.jcooper;


import java.util.Enumeration;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Vector;

import jcooper_support.InputFile;
import jcooper_support.Kid;
import jcooper_support.ParseVar;

/**
 * This team plays the role of Aggregate in the Iterator pattern
 * 
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class KidDataTeam {
	private Vector<Kid> _kids;

	/**
	 * This role is the Iterator participant in the Iterator pattern 
	 * @author Miguel P. Monteiro
	 */
	public class IteratorKidsFromClub implements Iterator<Kid> {
			private String _clubOfInterest; // name of club
			private Kid _nextKid = null; // next _nextKid to return
			private Enumeration<Kid> _kidsEnumerator; // gets all kids
	
			public IteratorKidsFromClub(String club) {
				_clubOfInterest = club; // save the club
				_kidsEnumerator = KidDataTeam.this.elements(); // get Enumerator
			}

			/**
			 * Return true if there are any more kids belonging to the specified
			 * club
			 **/
			public boolean hasNext() {
				boolean found = false;
				while (_kidsEnumerator.hasMoreElements() && !found) {
					_nextKid = _kidsEnumerator.nextElement();
					found = _nextKid.getClub().equals(_clubOfInterest);
				}
				if (!found)
					_nextKid = null; // set to null if none left
				return found;
			}
	
			public Kid next() {
				if (_nextKid != null)
					return _nextKid;
				// throw exception if access past end
				throw new NoSuchElementException();
			}
	
	//		@Override
			public void remove() {
				// TODO Auto-generated method stub
			}
		}

	public KidDataTeam(String filename) {
		_kids = new Vector<Kid>();
		InputFile f = new InputFile(filename);
		String s = f.readLine();
		while (s != null) {
			if (s.trim().length() > 0) {
				Kid k = new Kid(s);
				_kids.addElement(k);
			}
			s = f.readLine();
		}
	}

	public Kid[] getData() {
		Kid[] kd = new Kid[_kids.size()];
		for (int i = 0; i < _kids.size(); i++)
			kd[i] = (Kid) _kids.elementAt(i);
		return kd;
	}

	public int size() {
		return _kids.size();
	}

	public Kid getKid(int i) {
		return (Kid) _kids.elementAt(i);
	}

	public Vector<Object> getKidData(int key) {
		Vector<Object> v = new Vector<Object>();
		for (int i = 0; i < _kids.size(); i++)
			v.addElement(getKid(i).getData(key));
		return v;
	}

	public int getTableKey(String tabName) {
		int key = -1;
		tabName = tabName.toLowerCase();
		if (tabName.equals("frname"))
			key = ParseVar.FRNAME;
		if (tabName.equals("lname"))
			key = ParseVar.LNAME;
		if (tabName.equals("age"))
			key = ParseVar.AGE;
		if (tabName.equals("club"))
			key = ParseVar.CLUB;
		if (tabName.equals("time"))
			key = ParseVar.TIME;

		return key;
	}

	public String getTableName(int i) {
		String name = "";
		switch (i) {
		case ParseVar.FRNAME:
			name = "frname";
		case ParseVar.LNAME:
			name = "lname";
		case ParseVar.AGE:
			name = "age";
		case ParseVar.CLUB:
			name = "club";
		case ParseVar.TIME:
			name = "time";
		}
		return name;
	}

	public Enumeration<Kid> elements() {
		return _kids.elements();
	}
	public Iterator<Kid> getKidsInClubItt(String club) {
		return new IteratorKidsFromClub(club);
	}
}
