package mediator;

import java.util.Vector;

/**
 * Defines the general behavior of the Mediator design pattern.
 * 
 * Each concrete team of MediatorProtocol defines one kind of mediation
 * relationship. Within that kind of relationship, there can be any number of
 * Colleagues.
 * This reusable team does not use an explicit Mediator role, as it is played
 * by the team. The Mediator role is really about providing a context, which
 * is the natural function of teams. An explicit Mediator role would be
 * contrived in the context of OT/J. This way, the design is also simpler.
 * 
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 * @author Miguel P. Monteiro - improved on Gomes' initial variant
 */
public team class MediatorProtocol {
	/**
	 * Declares the Colleague role.
	 */
	protected class Colleague {
		public void changed() {
			updateMediator(this);
		}
	}

	/**
	 * Stores the mapping between Colleagues and Mediators. For each Colleague,
	 * its Mediator is stored.
	 */
	protected Vector<Colleague> _colleagues = new Vector<Colleague>();

	public void updateMediator(Colleague c) {
	}

	public void setColleague(Colleague colleague) {
		_colleagues.add(colleague);
	}
}
