package mediator.hk;

import javax.swing.*;
import org.objectteams.Team;
import java.awt.event.*;

/**
 * Implements the driver for the Mediator design pattern example.
 * 
 * Intent: Define an object that encapsulates how a set of objects interact.
 * Mediator promotes loose coupling by keeping objects from referring to each
 * other explicitly, and it lets you vary their interaction independently.
 * 
 * Participating objects are Buttons as Colleagues, and a Label as Mediator.
 * 
 * Every time an event of interest (a button click) occurs, the mediating
 * Label is updated and it in turn updates the respective calling
 * button.
 * Neither Mediator nor Colleagues have to be aware of their role within the
 * pattern.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
	static JFrame window4Mediator = new JFrame("Mediator Demo");
	static Button button1 = new Button("Button1");
	static Button button2 = new Button("Button2");
	static Label label = new Label("Click a button!");

	/**
	 * Implements the driver for the mediator example. It creates a small GUI
	 * with a label and two buttons. The buttons are Colleagues, the label is
	 * the Mediator.
	 * 
	 * Each button click causes the mediator to update itself and the calling
	 * button.
	 */
	public static void main(String[] args) {
		window4Mediator.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		JPanel panel = new JPanel();
		panel.add(label);
		panel.add(button1);
		panel.add(button2);

		window4Mediator.getContentPane().add(panel);
		window4Mediator.pack();
		window4Mediator.setVisible(true);

		MediatorTeam mediatorTeam = new MediatorTeam(label);
		mediatorTeam.activate(Team.ALL_THREADS);
		mediatorTeam.setColleague(button1);
		mediatorTeam.setColleague(button2);
	}
}
