package memento;

//import org.objectteams.Team;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class MementoConfinedProtocol {
	// optional role. To allow mementos to be kept inside the team
	protected abstract class CareTaker {
		public CareTaker() { }
		public void saveMemento(Confined memento) { }
		public void clearUndoList() { }
	}

	/**
	 * Defines the Originator role.
	 */
	protected abstract class Originator {
		/* sets the state to the state saved in Memento m */
		public abstract void setState(Memento m);

		/* returns an object with the current state of the Originator */
		protected abstract Object getState();

		/* creates a Memento with the current state of Originator */
		protected Memento createMemento() {
			return new Memento(this);
		}
	}

	protected abstract class Memento extends Confined {
		/**
		 * Sets the state of this Memento give the passed Originator.
		 * 
		 * @param Originator
		 *            o - the Originator with state to store
		 */
		public Memento(Originator o) { }

		/**
		 * Returns the state of this Memento give the passed originator.
		 * 
		 * @return the state stored by this Memento
		 */
		public abstract Object getState();
	}

	// team acts as caretaker
	protected Confined /*Memento*/ savedMemento;

	/**
	 * Creates a Memento object for an Originator in this case we
	 * return a confined role in order to maintain it enclosed in this team
	 * 
	 * @param o   the Originator to create a Memento for
	 */
	public void createMementoFor(Originator o) {
		this.savedMemento = o.createMemento();
	}
	public Confined getMementoFor(Originator o) {
		return o.createMemento();
	}

	/**
	 * Restores this Originator to a former state encapsulated in the
	 * Memento
	 * 
	 * @param o   the Originator to restore
	 */
	public void setMemento(Originator o) {
		o.setState((Memento) this.savedMemento);
	}
}