package memento;

//import org.objectteams.IConfined
//import org.objectteams.ITeam.IConfined;
/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class MementoProtocol {

	// optional role. To allow mementos to be kept inside the team
	protected abstract class CareTaker {
		public CareTaker() {
		}

		public void saveMemento(org.objectteams.ITeam.IConfined memento) {
		}

		public void clearUndoList() {
		}
	}

	/**
	 * Defines the Originator type. Used only internally.
	 */
	protected abstract class Originator {
		/* sets the state to the state saved in Memento m */
		public abstract void setState(Memento m);

		/* returns an object with the current state of the Originator */
		protected abstract Object getState();

		/* creates a Memento with the current state of Originator */
		public Memento createMemento() {
			return new Memento(this);
		}
	}

	protected abstract class Memento implements org.objectteams.ITeam.IConfined {
		/**
		 * Sets the state of this Memento give the passed Originator.
		 * @param Originator
		 *            o - the Originator with state to store
		 */
		public Memento(Originator o) {
		}

		/**
		 * Returns the state of this Memento give the passed originator.
		 * @return the state stored by this Memento
		 */
		public abstract Object getState();
	}

	/**
	 * Creates a Memento object for an Originator
	 * @param o   the Originator to create a Memento for
	 * @return the Memento storing the originator's state
	 */
	public Memento createMementoFor(Originator o) {
		return o.createMemento();
	}

	/**
	 * Restores this Originator to a former state encapsulated in the
	 * Memento
	 */
	public void setMemento(Originator o, org.objectteams.ITeam.IConfined m) {
		o.setState((Memento) m);
	}
}