package memento.hk.externalized;
/**
 * Implements the driver for the memento design pattern example. 
 *
 * Intent: Without violating encapsulation, capture and externalize an 
 * object's internal state so that the object can be restored to this state
 * later
 *
 * This example changes the state of the Originator five times, but
 * creates a Memento of it after the third change. After the 5 changes
 * are done, the Memento is used to restore the Originator's
 * state.
 * 
 * This implementation of Memento uses externalized roles.
 *
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
	public static void main(String[] args) {
		final MementoTeam mt = new MementoTeam();

//		mt.Memento o1 = mt.new Memento();
//		mt.Memento o2 = mt.new Memento();
//		Object obj = o1.getState();
//		if(o1.equals(o2))
//			System.out.println("equal");
//		else System.out.println("not equal");

		Counter counter = new Counter();
		//ought to be a caretaker!
		Memento<@mt> memento = null;

		for (Integer i=1; i<=5; i++) {
			counter.increment();
			counter.show(); 
			if (i == 2) {
				memento = mt.createMementoFor(counter);
				//Methods inherited from Object are still allowed:
				memento.equals(new Object());
			}
		}
		Memento<@mt> memento2 = memento; 
		System.out.println("\nTrying to reinstate state (2)...");
		mt.setMemento(counter, memento2);
		counter.show();
	}
}