package memento.hk.externalized2;
/**
 * Implements the driver for the memento design pattern example. 
 *
 * Intent: Without violating encapsulation, capture and externalize an 
 * object's internal state so that the object can be restored to this state
 * later
 *
 * This example changes the state of the Originator five times, but
 * creates a Memento of it after the third change. After the 5 changes
 * are done, the Memento is used to restore the Originator's
 * state.
 * 
 * This implementation of Memento uses externalized roles.
 *
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
	public static void main(String[] args) {
		final MementoTeam mementoTeam = new MementoTeam();

		//Unlike in externalized, the call to equals below originates a compiler error:
//		mementoTeam.Memento o1 = mementoTeam.new Memento();
//		mementoTeam.Memento o2 = mementoTeam.new Memento();
//		if(o1.equals(o2))
//			System.out.println("equal");
//		else System.out.println("not equal");

		Counter counter = new Counter();
		//ought to be a caretaker!
		Memento<@mementoTeam> memento = null;
		for (Integer i=1; i<=5; i++) {
			counter.increment();
			counter.show(); 
			if (i == 2) {
				memento = mementoTeam.createMementoFor(counter);
			}
		}
		//also ilegal:
//		System.out.println("\nTrying to reinstate state (" + memento.getState() + ")...");
//		System.out.println("\nTrying to reinstate state (" + memento + ")...");
		System.out.println("\nTrying to reinstate state (2)...");
		mementoTeam.setMemento(counter, memento);
		counter.show();
	}
}