package memento.hk.externalized2;

import memento.MementoConfinedProtocol;
/**
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class MementoTeam extends MementoConfinedProtocol {

	protected class Originator playedBy Counter {
		/* returns an object with the current state of the Originator */
		protected abstract Object getState();
		getState -> getCurrentValue;

		void setState(Memento m) -> void setCurrentValue(int value) with {
			(Integer) m.getState() -> value
		}

		/* creates a Memento with the current state of Originator */
		protected Memento createMemento() {
			return new Memento(this);
		}
	}

	public class Memento extends Confined {
		private Object state;

		protected Memento(Originator o) {
			this.state = o.getState();
		}

		protected Object getState() {
			return state;
		}
	}

	/**
	 * Creates a Memento object for an Originator
	 * 
	 * @param o
	 *            the Originator to create a Memento for
	 * @return the Memento storing the originator's state
	 */
	public Memento createMementoFor(Counter as Originator o) {
		return o.createMemento();
	}

	/**
	 * Restores this Originator to a former state encapsulated in the Memento
	 */
	public void setMemento(Counter as Originator o, Memento m) {
		o.setState(m);
	}
}
