package memento.hk.opaque;

/**
 * Implements the driver for the memento design pattern example. 
 *
 * Intent: Without violating encapsulation, capture and externalize an 
 * object's internal state so that the object can be restored to this state
 * later
 * This example changes the state of the Originator five times, but
 * creates a Memento of it after the third change. After the 5 changes
 * are done, the Memento is used to restore the Originator's
 * state.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
    /**
     * This example changes the state of the Originator five times, but
     * creates a Memento of it after the third change. After the 5 
     * changes are done, the Memento is used to restore the 
     * Originator's state.  
     */
	public static void main(String[] args) {
		final OpaqueMementoTeam omt = new OpaqueMementoTeam();

//		omt.Memento o1 = omt.new Memento();
//		omt.Memento o2 = omt.new Memento();
//		if(o1.equals(o2))
//			System.out.println("equal");
//		else System.out.println("not equal");

		Counter counter = new Counter();
		for (int i=1; i<=5; i++) {
			counter.increment();
			counter.show(); 
			if (i==3) {
				omt.saveMemento(counter);
			}
		}
		System.out.println("\nTrying to reinstate state (3)...");
		omt.restoreMemento(counter);
		counter.show();
	}
}