package memento.jcooper;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Vector;
import javax.swing.JPanel;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Mediator {
	private boolean _startRect;
	private boolean _rectSelected;
	private Vector<VisRectangle> _drawings;
	private RectButton _rect;
	private JPanel _canvas;
	private VisRectangle _selectedRectangle;

	public Mediator() {
		_startRect = false;
		_rectSelected = false;
		_drawings = new Vector<VisRectangle>();
	}

	public void startRectangle() {
		_startRect = true;
	}

	// to be weaved by the team for undo purposes
	public void newRect(int x, int y) {
		VisRectangle v = new VisRectangle(x, y);
		_drawings.addElement(v); // add new element to list
		_startRect = false; // done with this rectangle
		_rect.setSelected(false); // unclick button
		_canvas.repaint();
	}

	public void createRect(int x, int y) {
		unpick(); // make sure no rectangle is selected
		// if _rect button is depressed
		if (_startRect) {
			newRect(x, y);
		} else
			pickRect(x, y); // if not pressed look for _rect to select
	}

	public void registerRectButton(RectButton rb) {
		_rect = rb;
	}

	public void registerCanvas(JPanel p) {
		_canvas = p;
	}

	private void unpick() {
		_rectSelected = false;
		if (_selectedRectangle != null) {
			_selectedRectangle.setSelected(false);
			_selectedRectangle = null;
			repaint();
		}
	}

	public void rememberPosition() {
	}

	public void pickRect(int x, int y) {
		// save current selected rectangle to avoid double save of undo
		VisRectangle lastPick = _selectedRectangle;
		unpick();
		for (int i = 0; i < _drawings.size(); i++) {
			VisRectangle v = (VisRectangle) _drawings.elementAt(i);
			// did click inside a rectangle
			if (v.contains(x, y)) {
				_selectedRectangle = v; // save it
				_rectSelected = true;
				if (_selectedRectangle != lastPick) // but don't save twice
					rememberPosition();
				v.setSelected(true); // turn on handles
				repaint(); // and redraw
			}
		}
	}

	public void clear() {
		_drawings = new Vector<VisRectangle>();
		_rectSelected = false;
		_selectedRectangle = null;
		repaint();
	}

	private void repaint() {
		_canvas.repaint();
	}

	public void drag(int x, int y) {
		if (_rectSelected) {
			if (_selectedRectangle.contains(x, y)) {
				_selectedRectangle.move(x, y);
				repaint();
			}
		}
	}

	public void reDraw(Graphics g) {
		g.setColor(Color.black);
		for (int i = 0; i < _drawings.size(); i++) {
			VisRectangle v = (VisRectangle) _drawings.elementAt(i);
			v.draw(g);
		}
	}

	public void undo() {
	}
}
