package memento.jcooper;

import memento.MementoProtocol;
import java.util.Vector;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class MementoTeam extends MementoProtocol {

	public void clearUndoList(Mediator as CareTaker c) {
		c.clearUndoList();
	}

	protected class CareTaker playedBy Mediator {
		void repaint() -> void repaint();
		Vector<VisRectangle> getDrawings() -> get Vector<VisRectangle> _drawings;
		VisRectangle getSelectedRectangle() -> get VisRectangle _selectedRectangle;
		boolean getRectSelected() -> get boolean _rectSelected;
		Vector<org.objectteams.ITeam.IConfined> _undoList;
		undo <- after undo;
		saveNewRectRequest <- after newRect;
		clearUndoList <- before clear;
		saveOldRectPosition <- after rememberPosition;

		public void undo() {
			if (_undoList.size() > 0) {
				// get last element in undo list
				org.objectteams.ITeam.IConfined memento = _undoList.lastElement();
				_undoList.removeElementAt(_undoList.size() - 1); // and remove it
				// if this null, the last action was a new rectangle
				if (memento != null) {
					setMemento(memento);
				} else {
					// remove last created rectangle
					Object drawObj = getDrawings().lastElement();
					getDrawings().removeElement(drawObj);
					repaint();
				}
				repaint();
			}
		}

		public CareTaker(Mediator m) {
			clearUndoList();
		}

		// empty the undo list
		public void clearUndoList() {
			this._undoList = new Vector<org.objectteams.ITeam.IConfined>();
		}

		public void saveOldRectPosition() {
			if (getRectSelected()) {
				org.objectteams.ITeam.IConfined memento =
					createMementoFor(getSelectedRectangle());
				_undoList.addElement(memento);
			}
		}

		public void saveNewRectRequest() {
			_undoList.addElement(null);
		}
	}

	protected class Originator implements ILowerable playedBy VisRectangle {
		void set_y(int _y) -> set int _y;
		void set_x(int _x) -> set int _x;

		public void setState(Memento m) {
			VisRectangle tmp = (VisRectangle) m.getState();
			set_x(tmp._x);
			set_y(tmp._y);
		}

		//returns an instance of the base type, to be used by the memento
		protected Object getState() {
			return this.lower();
		}
	}

	protected class Memento implements org.objectteams.ITeam.IConfined {
		VisRectangle savedState;
		VisRectangle originator;

		public Memento(Originator o) {
			this.originator = o;
			VisRectangle _rect = (VisRectangle) o.getState();
			savedState = new VisRectangle(_rect._x, _rect._y);
		}

		public Object getState() {
			return savedState;
		}

		public VisRectangle getOriginator() {
			return originator;
		}
	}

	public org.objectteams.ITeam.IConfined createMementoFor(VisRectangle as Originator o) {
		return super.createMementoFor(o);
	}

	public void setMemento(VisRectangle as Originator o, org.objectteams.ITeam.IConfined m) {
		super.setMemento(o, m);
	}

	public void setMemento(org.objectteams.ITeam.IConfined m) {
		VisRectangle o = ((Memento) m).getOriginator();
		this.setMemento(o, m);
	}
}
