package memento.jcooper;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import jcooper_support.ICommand;
import jcooper_support.JxFrame;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
class MouseApp extends MouseAdapter {
	private Mediator med;

	public MouseApp(Mediator md) {
		super();
		med = md;
	}

	public void mousePressed(MouseEvent e) {
		med.createRect(e.getX(), e.getY());
	}

	public void mouseReleased(MouseEvent e) {
		med.rememberPosition();
	}
}

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
class MouseMoveApp extends MouseMotionAdapter {
	private Mediator med;

	public MouseMoveApp(Mediator md) {
		super();
		med = md;
	}

	public void mouseDragged(MouseEvent e) {
		med.drag(e.getX(), e.getY());
	}
}

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Window4Memento extends JxFrame implements ActionListener {
	private static final long serialVersionUID = 0L;

	private JToolBar tbar;
	private Mediator med;

	public Window4Memento() {
		super("Memento Drawing");
		JPanel jp = new JPanel();
		getContentPane().add(jp);

		med = new Mediator();
		jp.setLayout(new BorderLayout());
		tbar = new JToolBar();
		jp.add("North", tbar);
		RectButton rect = new RectButton(this, med);
		tbar.add(rect);
		UndoButton undo = new UndoButton(this, med);
		tbar.add(undo);
		tbar.addSeparator();
		ClearButton clr = new ClearButton(this, med);
		tbar.add(clr);
		JCanvas canvas = new JCanvas(med);
		jp.add("Center", canvas);
		MouseApp map = new MouseApp(med);
		canvas.addMouseListener(map);
		MouseMoveApp mvap = new MouseMoveApp(med);
		canvas.addMouseMotionListener(mvap);
		setSize(new Dimension(400, 300));
		setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		ICommand comd = (ICommand) e.getSource();
		comd.execute();
	}
}
