package observer.hk;

/**
 * Provides a means to output messages. Objects of this class act as output
 * devices.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Screen {

	/**
	 * the individual name of this screen object
	 */
	private String name;

	/**
	 * creates a new Screen object with the provided name.
	 * 
	 * @param _name
	 *            the name for the new Screen object
	 */

	public Screen(String s) {
		this.name = s;
	}

	/**
	 * Prints the name of the Screen object and the argument string
	 * to stdout.
	 * 
	 * @param s
	 *            the string to print
	 */
	public void display(String s) {
		System.out.println(name + ": " + s);

	}

	/**
	 * Updates an <i>Observer</i>. Uses the <i>push</i> strategy (i.e. the
	 * subject triggering the update passes itself as an argument).
	 * 
	 * This particular method prints a message showing what object caused the
	 * update
	 * 
	 * @param s
	 *            the <i>Subject</i> triggering the update
	 */
	public void refresh(Object s) {
		String subjectTypeName = s.getClass().getName();
		subjectTypeName = subjectTypeName.substring(subjectTypeName
				.lastIndexOf(".") + 1, subjectTypeName.length());
		display("update received from a " + subjectTypeName + " object");
	}
}
