package observer.jcooper;

import observer.ObserverProtocol;
import java.awt.event.ItemEvent;
import java.awt.Color;
import javax.swing.JRadioButton;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class ConcreteObserverTeam extends ObserverProtocol {

	protected class Subject playedBy Window4Observer{		
		private String selectedColor = "none";

		public String getSelectedColor() {
			return selectedColor;
		}

		public void changeRadioButton(ItemEvent e) 
		//Guard predicate: if enabled, blocks callin when subject is in "Red" state)
//		when (((JRadioButton)e.getSource()).getText() != "Red")
		{
			if (e.getStateChange() == ItemEvent.SELECTED){
				selectedColor = ((JRadioButton)e.getSource()).getText();
				System.out.println(selectedColor);
				this.notifyObservers();
			}
		}
		changeRadioButton <- before itemStateChanged;
	}

	//Obtain String from the chain_of_responsibility chain of subject
	private String getColorFromSubject(Subject s){ return s.getSelectedColor(); }

	protected class ColorFrameObserver extends Observer playedBy ColorFrame {		
		void update(Subject s) -> void changeColor(String selectedColor) with {
			getColorFromSubject(s) -> selectedColor
		}

		private abstract Color getColorFromColorFrame();
		getColorFromColorFrame -> get _color;

		private abstract void setColorFromColorFrame(Color c);
		setColorFromColorFrame -> set _color;
	}

	protected class ColorListObserver extends Observer playedBy ColorList {
		void update(Subject s) -> void writeColorOnWindow(String s) with {
			getColorFromSubject(s)-> s
		}
	}

	//Declared lifting: uses AnyBase to represent any Observer class
	public <AnyBase base Observer>
	void addObserver(AnyBase as Observer obs, Window4Observer as Subject sub){
		sub.addObserver(obs);
	}

	public <AnyBase base Observer>
	void remObserver(AnyBase as Observer obs, Window4Observer as Subject sub){
		sub.removeObserver(obs);
	}
}
