package observer.jcooper;

import org.objectteams.Team;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
	private static void printHelpMessage() {
		System.out.println("Observer design pattern");
		System.out.println();
		System.out.println("Intent: define a one-to-many dependency between objects");
		System.out.println("so that when one object changes state, all its dependents");
		System.out.println("are notified and updated automatically.");
		System.out.println();
		System.out.println("In this example, the Subject is the window with radio buttons,");
		System.out.println("through which you can select one of three colours.");
		System.out.println("The Observers are the remaining windows: color and list frames.");
	}

	public static void main(String[] args) {
		printHelpMessage();
		ConcreteObserverTeam observationTeam = new ConcreteObserverTeam();
		observationTeam.activate(Team.ALL_THREADS);

		// subject
		Window4Observer mainWindow = new Window4Observer();

		// observers
		ColorFrame colorFrame1 = new ColorFrame();
		ColorFrame colorFrame2 = new ColorFrame();
		ColorFrame colorFrame3 = new ColorFrame();
		ColorList colorList = new ColorList();

		observationTeam.addObserver(colorFrame1, mainWindow);
		observationTeam.addObserver(colorFrame2, mainWindow);
		observationTeam.addObserver(colorFrame3, mainWindow);
		observationTeam.addObserver(colorList, mainWindow);
	}

}
