package prototype;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class PrototypeProtocol {
	/**
	 * prototype role
	 */
	protected abstract class PrototypeR {
		/**
		 * Attaches a default clone() method to all prototypes.
		 * This makes use of Java's clone() mechanism that creates a deep copy
		 * of the object in question.
		 * 
		 * @return a copy of the object
		 */
		public Object clone() throws CloneNotSupportedException {
			return super.clone();
		}
		public abstract Object deepClone();
	}

	/**
	 * Provides a static default team method for cloning prototypes. It uses the
	 * attached clone() method if possible. If not, it calls the static
	 * createCloneFor(Prototype) method.
	 * 
	 * @param object the prototype object to clone
	 * @return a copy of the object
	 */
	public Object createClone(PrototypeR object) {
		try {
			return object.clone();
		} catch (CloneNotSupportedException ex) {
			return createDeepClone(object);
		}
	}

	/**
	 * Provides an alternative method for cases when the default
	 * clone() method fails. This method can be overwritten by
	 * concrete subaspects. In this default implementation it returns null.
	 * 
	 * @param object
	 *            the prototype object to clone
	 * @return a copy of the object, but null in this case
	 */
	public Object createDeepClone(PrototypeR object) {
		return null;
	}
}
