/*
   2009/09/23
 */
package prototype.jcooper;

import jcooper_support.Support;

import org.objectteams.Team;


/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
	private static void printHelpMessage() {
		System.out.println("Prototype design pattern");
		System.out.println();
		System.out.println("Intent: specify the kinds of objects to create using a prototypical");
		System.out.println("instance, and create new objects by copying this prototype.");
		System.out.println();
		System.out.println("Consider a database of a large number of swimmers. Each swimmer");
		System.out.println("swims several strokes and distances throughout a season.");
		System.out.println("The best times for swimmers are tabulated by age group, and");
		System.out.println("many swimmers will have birthdays and fall into new age groups");
		System.out.println("within a single season. Thus the query to determine which swimmers");
		System.out.println("did the best in their age group that season is dependent on the");
		System.out.println("date of each meet and on each swimmers birthday.");
		System.out.println("Therefore, the computational cost of assembling this table is high.");
		System.out.println("Once we have this table sorted by sex, we could imagine wanting it");
		System.out.println("sorted just by time, or by actual age rather than by age group");
		System.out.println("It would not be sensible to recompute these data, so some sort of");
		System.out.println("copy is desirable.");
		System.out.println();
		System.out.println("In the original class (left), names are sorted by sex and then by time.");
		System.out.println("In the cloned class (right), they are sorted only by time.");
		System.out.println("Click on button 'Clone' to create the newly sorted clone.");
		System.out.println("Clicking on button 'Refresh' reloads the left list from the original data.");
		System.out.println("\nNOTE: unlike in the original version, the names in the left list are NOT");
		System.out.println("re-sorted because the team performs a deep copy instead of a shallow copy.");
	}

	public static void main(String[] args) {
		printHelpMessage();
		SwimPrototypeTeam swimPrototypeTeam = new SwimPrototypeTeam();
		swimPrototypeTeam.activate(Team.ALL_THREADS);
		new Window4Prototype(Support.FOLDER_LOCATION + "\\" + "Swimmers.txt", swimPrototypeTeam);
	}
}
