package prototype.jcooper;

import prototype.PrototypeProtocol;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.Vector;

/**
 * This version differs from the original Java version in that team
 * SwimPrototypeTeam performs a deep copy of SwimData instead of a
 * shallow copy.
 * 
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class SwimPrototypeTeam extends PrototypeProtocol implements
		Serializable {
	private static final long serialVersionUID = 1L;

	public class SwimmerPrototype extends PrototypeR implements ILowerable,
			Serializable playedBy SwimData {
		Vector<Swimmer> getSwimmers() -> get Vector<Swimmer> swimmers;

		private static final long serialVersionUID = 1L;

		public Object clone() throws CloneNotSupportedException {
			return super.clone();
		}

		public Object deepClone() {
			try {
				ByteArrayOutputStream b = new ByteArrayOutputStream();
				// ObjectOutputStream out = writeTo(b);//JG: remove this from SwimData
				ObjectOutputStream out = new ObjectOutputStream(b);
				out.writeObject(this);
				//JG: Why doesn't work?
				ByteArrayInputStream bIn = new ByteArrayInputStream(b.toByteArray());
				ObjectInputStream oi = new ObjectInputStream(bIn);
				return (oi.readObject());
			} catch (Exception e) {
//				System.err.println("Exception raised:" + e.getMessage());
//				e.printStackTrace();

				//JG: This way it works, but I would like to use the original approach
				//MPM: I suspect the cause of the problem is that teams, and specially
				//roles as virtual classes, do not work with the Serializable mechanism.
				SwimData stopgapSwimmers = new SwimData();
				stopgapSwimmers.swimmers = new Vector<Swimmer>(getSwimmers());
				// return null;
				return stopgapSwimmers;
			}
		}
	}

	public Object createClone(SwimData as SwimmerPrototype object) {
		try {
			return object.clone();
		} catch (CloneNotSupportedException ex) {
			return object.deepClone();
		}
	}
}
