package proxy.hk;

/**
 * Implements the driver for the Proxy design pattern example.
 * 
 * Intent: Provide a surrogate or placeholder for another object to control
 * access to it.
 * 
 * Participating objects are OutputImplementation as RealSubject and 
 * RequestCounter and RequestBlocker as "Proxy teams".
 * 
 * ProxyTeams needs not to implement all methods of OutputSubject, only weave
 * those which it is interested in.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {

	/**
	 * Implements the driver for the proxy design pattern.
	 */
	public static void main(String[] args) {
		OutputImplementation real = new OutputImplementation();
		RequestCounterTeam countingProxy = new RequestCounterTeam();
		countingProxy.activate();
		RequestBlockerTeam blockingProxy = new RequestBlockerTeam();
		blockingProxy.activate();

		System.out.println("\n===> Issuing SAFE request...");
		real.safeRequest("Safe Request");
		System.out.println("\n===> Issuing REGULAR request...");
		real.regularRequest("Normal Request");
		System.out.println("\n===> Issuing UNSAFE request...");
		real.unsafeRequest("Unsafe Request");

		System.out.println("\n===> Issuing SAFE request...");
		real.safeRequest("Safe Reqeust");
		System.out.println("\n===> Issuing REGULAR request...");
		real.regularRequest("Normal Request");
		System.out.println("\n===> Issuing UNSAFE request...");
		real.unsafeRequest("Unsafe Request");
	}
}
