package singleton.hk;

/**
 * Implements a sample Singleton  class. The class's functionality is to
 * store an instance-specific ID and provide a print() method
 * that shows an object's ID.
 * Note that in this implementation the Singleton
 * class has to know that it implements the patter
 * (i.e. has to have appropriate code in it).
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class PrinterSingleton {
	/**
	 * counts the instances of this class
	 */
	protected static int objectsSoFar = 0;

	/**
	 * stores this Singleton 's only instance
	 */
	protected static PrinterSingleton onlyInstance = null;

	/**
	 * each instance has an ID to distinguish them.
	 */
	protected int id;

	/**
	 * Creates a new PrinterSingleton. The new instance gets an ID equal to the
	 * total number of instances created of that type. This constructor is
	 * protected to disallow it being called from other places but the factory
	 * method and this type's subtypes. Unfortunately, this still allows other
	 * types in the same package to access the constructor. Choosing to make it
	 * private would prevent that problem, but would make it impossible to
	 * subclass the type properly (as subtypes then could not use super(..) in
	 * their constructor.
	 */
	protected PrinterSingleton() {
		id = ++objectsSoFar;
	}

	/**
	 * Factory method that provides access to the Singleton instance. Uses
	 * creation-on-demand.
	 * 
	 * @return the unique Singleton instance
	 */
	public static PrinterSingleton instance() {
		if (onlyInstance == null) {
			onlyInstance = new PrinterSingleton();
		}
		return onlyInstance;
	}

	/**
	 * Prints the instance's ID to System.out.
	 */
	public void print() {
		System.out.println("\tMy ID is " + id);
	}
}
