/*
   2009/09/22
 */
package singleton.jcooper;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 */
public class Main {
	private static void printHelpMessage() {
		System.out.println("Design pattern Singleton");
		System.out.println();
		System.out.println("Intent: Ensure a class only has one instance, and provide a global point of access to it.");
	}

	private static void test_iSpooler() {
		iSpooler pr1, pr2;
		// open one printer--this should always work
		System.out.println("Opening one spooler");
		pr1 = iSpooler.Instance();
		if (pr1 != null)
			System.out.println("got 1 spooler");
		// try to open another printer --should fail
		System.out.println("Opening two spoolers");

		pr2 = iSpooler.Instance();
		if (pr2 == null)
			System.out.println("no instance available");
		// fails because constructor is privatized
		// singleSpooler pr3 = new singleSpooler();
	}

	private static void testSinglePrinter() {
		// Printer pr1, pr2;
		// open one printer--this should always work
		System.out.println("Opening one printer");
		try {
			/* pr1 = */new Printer();
		} catch (SingletonException e) {
			System.err.println(e.getMessage());
		}
		// try to open another printer --should fail
		System.out.println("Opening two printers");
		try {
			/* pr2 = */new Printer();
		} catch (SingletonException e) {
			System.err.println(e.getMessage());
		}
	}

	private static void testSingleSpooler() {
		// SingleSpooler pr1, pr2;
		// open one printer--this should always work
		System.out.println("Opening one spooler");
		try {
			/* pr1 = */new SingleSpooler();
		} catch (SingletonException e) {
			System.out.println(e.getMessage());
		}
		// try to open another printer --should fail
		System.out.println("Opening two spoolers");
		try {
			/* pr2 = */new SingleSpooler();
		} catch (SingletonException e) {
			System.out.println(e.getMessage());
		}
	}

	public static void main(String[] args) {
		printHelpMessage();
		// Select the scenario you want to run, comment out the others:
//		test_iSpooler();
//		testSinglePrinter();
		testSingleSpooler();
	}

}
