package state.hk;

/**
 * Implements the driver for the State design pattern example.
 *
 * Intent: Allow an object to alter its behavior when its internal state
 * changes. The object will appear to change its class
 * *
 * This example of the State design pattern models a Queue ADT with
 * a limited capacity that has three different states: 
 * 
 * Empty:	The queue is empty 
 * Normal:  The queue is neither empty nor full
 * Full:	The queue is full (# of elements = capacity)
 * 
 * The queue stores Objects. The following operations are defined on Queue:
 * 
 * insert(Object) Inserts a new Object into the queue
 * getFirst():Object Returns the first element in the queue
 * removeFirst() Removes the first elelemts from the queue 
 */
public class Main { 

    /**
     * Implements insertion into a queue. Prints out status messages.
     *
     * @param queue the queue to insert into
     * @param s the string to insert into the queue
     */
    public static void testInsert(Queue queue, String s) {
		System.out.print("   Trying to insert ["+s+"] into the queue ... ");
		boolean status = queue.insert(s);
		if (status == true) {
			System.out.println("successful"); 
		} else {
			System.out.println("NOT successful, queue probably full");
		}
	}

    /**
     * Implements deletion from a queue. Prints out status messages.
     *
     * @param queue the queue to delete items from
     */     	
	public static void testRemove(Queue queue) {
		System.out.print("   Trying to remove 1st element of the queue ... ");
		String item = (String) queue.getFirst(); 
		boolean status = queue.removeFirst();
		if (status == true) {
			System.out.println("successful: "+item); 
		} else {
			System.out.println("NOT successful: "+item);
		}
	}

    /**
     * Implements the driver for the State design pattern example.
     *
     * @param args the command line parameters, unused
     */
	public static void main(String[] args) {
		System.out.println(" OT/J Testing Pattern: State - STARTING\n");

		Queue queue = new Queue(new StateTeam());
		testInsert(queue, "This ");
		testInsert(queue, "is ");  
		testInsert(queue, "a ");
		testInsert(queue, "test");   
		System.out.println();
		testRemove(queue);
		testRemove(queue);
		testRemove(queue);
		testRemove(queue);

		System.out.println("\nTesting Pattern: State - FINISHED");
	}
}
