package state.jcooper;

/**
 * The original JCooper example uses two sub-classes of this class,
 * for keeping backups of the state of visCircle and visRect instances. 
 * Since it turned out that the backed-up state was identical in both
 * cases and applicable to super-class Drawing, the duplication was
 * removed giving rise to this single Memento class, which serves as
 * memento for both VisibleRectangle and VisibleCircle.
 *  
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 */
public class DrawingMemento {
	/* 
	 * Can be VisibleRectangle or VisibleCircle
	 */
	protected Drawing _drawing;

	/*
	 * Saved state, common to VisibleRectangle and VisibleCircle
	 */
	protected int x, y, w, h; 

	public DrawingMemento(Drawing drawing) {
		_drawing = drawing;
		x = _drawing.x;
		y = _drawing.y;
		w = _drawing.w;
		h = _drawing.h;
	}

	public void restore() {
		_drawing.x = x;
		_drawing.y = y;
		_drawing.h = h;
		_drawing.w = w;
	}
}
