package state.jcooper;

import java.awt.*;
import java.awt.event.ActionListener;
import javax.swing.JToggleButton;

import jcooper_support.ICommand;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 */
public class FillButton extends JToggleButton implements ICommand {
	private static final long serialVersionUID = 0L;

	private Mediator _mediator;

	public FillButton(ActionListener act, Mediator mediator) {
		super(" ");
		setSize(new Dimension(35, 35));
		setBorderPainted(true);
		setMargin(new Insets(5, 12, 5, 12));
		setToolTipText("Fill rectangle");
		addActionListener(act);
		_mediator = mediator;
		_mediator.registerFillButton(this);
	}

	public Dimension getPreferredSize() {
		return new Dimension(35, 35);
	}

	public void paint(Graphics g) {
		super.paint(g);
		int h = getHeight();
		int w = getWidth();
		g.setColor(Color.black);
		g.drawRect(4, 4, w - 8, h - 8);
		g.setColor(Color.red);
		g.fillRect(4, 10, w - 8, h - 14);
	}

	public void execute() {
		if (isSelected()) {
			_mediator.startFill();
		}
	}
}
