/*
   12 de Ago de 2010
*/
package state.jcooper;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Vector;

public class Mediator {
	private boolean _drawSelected;
	private Vector<Drawing> _drawingsVector;
	private Vector<DrawingMemento> _undoVector;
	private RectButton _rectButton;
	private FillButton _fillButton;
	private CircleButton _circleButton;
	private PickButton _pickButton;
	private JCanvas _panelCanvas;
	private Drawing _selectedDrawing;
	private StateTeam _stateManager;

	public Mediator() {
		// startRect = false;
		_drawSelected = false;
		_drawingsVector = new Vector<Drawing>();
		_undoVector = new Vector<DrawingMemento>();
		_stateManager = new StateTeam(this);
	}

	public void startRectangle() {
		_stateManager.setRectangle();
		_pickButton.setSelected(false);
		_circleButton.setSelected(false);
		_fillButton.setSelected(false);
	}

	public void startCircle() {
		_stateManager.setCircle();
		_rectButton.setSelected(false);
		_pickButton.setSelected(false);
		_fillButton.setSelected(false);
	}

	public void startFill() {
		_stateManager.setFill();
		_rectButton.setSelected(false);
		_circleButton.setSelected(false);
		_pickButton.setSelected(false);
		_stateManager.select(_selectedDrawing, Color.red);
		repaint();
	}

	public void startArrow() {
		_stateManager.setArrow();
		_rectButton.setSelected(false);
		_circleButton.setSelected(false);
		_fillButton.setSelected(false);
	}

	public Drawing getSelected() {
		return _selectedDrawing;
	}

	public void setSelected(Drawing drawing) {
		if (drawing != null) {
			_drawSelected = true;
			_selectedDrawing = drawing;
			drawing.setSelected(true);
			repaint();
		}
	}

	public void fillSelected() {
		if (_drawSelected) {
			_selectedDrawing.setFill(Color.red);
		}
	}

	public Vector<Drawing> getDrawings() {
		return _drawingsVector;
	}

	public void addDrawing(Drawing drawing) {
		_drawingsVector.addElement(drawing);
	}

	public void registerRectButton(RectButton rectangleButton) {
		_rectButton = rectangleButton;
	}

	public void registerCircleButton(CircleButton circleButton) {
		_circleButton = circleButton;
	}

	public void registerPickButton(PickButton pickButton) {
		_pickButton = pickButton;
	}

	public void registerFillButton(FillButton fillButton) {
		_fillButton = fillButton;
	}

	public void registerCanvas(JCanvas canvas) {
		_panelCanvas = canvas;
	}

	public void mouseDown(int x, int y) {
		_stateManager.mouseDown(x, y);
		repaint();
	}

	public void mouseUp(int x, int y) {
		_stateManager.mouseUp(x, y);
	}

	public void unpick() {
		_drawSelected = false;
		if (_selectedDrawing != null) {
			_selectedDrawing.setSelected(false);
			_selectedDrawing = null;
			repaint();
		}
	}

	public void rememberPosition() {
		if (_drawSelected) {
			DrawingMemento memento = new DrawingMemento(_selectedDrawing);
			_undoVector.addElement(memento);
		}
	}

	public void pickRect(int x, int y) {
	}

	public void clear() {
		_drawingsVector = new Vector<Drawing>();
		_undoVector = new Vector<DrawingMemento>();
		_drawSelected = false;
		_selectedDrawing = null;
		repaint();
	}

	private void repaint() {
		_panelCanvas.repaint();
	}

	public void mouseDrag(int x, int y) {
		_stateManager.mouseDrag(x, y);
	}

	public void reDraw(Graphics g) {
		g.setColor(Color.black);
		for (int i = 0; i < _drawingsVector.size(); i++) {
			Drawing v = (Drawing) _drawingsVector.elementAt(i);
			v.draw(g);
		}
	}

	public void undo() {
		if (_undoVector.size() > 0) {
			// get last element in undo list
			Object obj = _undoVector.lastElement();
			_undoVector.removeElement(obj); // and remove it

			//MPM: below, 'Switch Statement' bad code smell!

			// if this is an Integer, the last action was a new rectangle
			if (obj instanceof Integer) {
				// remove last created rectangle
				Object drawObj = _drawingsVector.lastElement();
				_drawingsVector.removeElement(drawObj);
			}
			// if this is a Memento, the last action was a move
			if (obj instanceof DrawingMemento) {
				// get the Memento
				DrawingMemento memento = (DrawingMemento) obj;
				memento.restore(); // and restore the old position
			}
			repaint();
		}
	}
}
