/*
   12 de Ago de 2010
*/
package state.jcooper;

import java.awt.Color;
import java.util.Vector;

public team class StateTeam {
	private State _currentState;
	private RectState _rectangleState;
	private ArrowState _arrowState;
	private CircleState _circleState;
	private FillState _fillState;

	public StateTeam(Mediator mediator) {
		_rectangleState = new RectState(mediator);
		_circleState = new CircleState(mediator);
		_arrowState = new ArrowState(mediator);
		_fillState = new FillState(mediator);
		_currentState = _arrowState;
	}

	protected abstract class State {
		public void mouseDown(int x, int y) { }
		public void mouseUp(int x, int y) { }
		public void mouseDrag(int x, int y) { }
		public void select(Drawing d, Color c) { }
	}

	protected class ArrowState extends State {
		private Mediator _mediator;

		public ArrowState(Mediator mediator) {
			_mediator = mediator;
		}

		public void mouseDown(int x, int y) {
			Vector<Drawing> drawings = _mediator.getDrawings();
			for (int i = 0; i < drawings.size(); i++) {
				Drawing d = (Drawing) drawings.elementAt(i);
				if (d.contains(x, y))
					_mediator.setSelected(d);
			}
		}
	}

	protected class CircleState extends State {
		private Mediator _mediator; // save Mediator

		public CircleState(Mediator mediator) {
			_mediator = mediator;
		}

		// Draw circle where mouse clicks
		public void mouseDown(int x, int y) {
			_mediator.addDrawing(new VisibleCircle(x, y));
		}
	}

	protected class FillState extends State {
		private Color _color; // save current color
		private Mediator _mediator; // save Mediator

		public FillState(Mediator mediator) {
			_mediator = mediator;
		}

		/* Fill drawing if selected */
		public void select(Drawing drawing, Color color) {
			_color = color;
			if (drawing != null) {
				drawing.setFill(color); // fill that drawing
			}
		}

		/* Fill drawing if you click inside one */
		public void mouseDown(int x, int y) {
			Vector<Drawing> drawings = _mediator.getDrawings();
			for (int i = 0; i < drawings.size(); i++) {
				Drawing d = (Drawing) drawings.elementAt(i);
				if (d.contains(x, y))
					d.setFill(_color); // fill drawing
			}
		}
	}

	protected class RectState extends State {
		private Mediator _mediator; // save the Mediator here

		public RectState(Mediator mediator) {
			_mediator = mediator;
		}

		// create a new Rectangle where mode clicks
		public void mouseDown(int x, int y) {
			_mediator.addDrawing(new VisibleRectangle(x, y));
		}
	}

	public void setRectangle() {
		_currentState = _rectangleState;
	}

	public void setCircle() {
		_currentState = _circleState;
	}

	public void setFill() {
		_currentState = _fillState;
	}

	public void setArrow() {
		_currentState = _arrowState;
	}

	public void mouseDown(int x, int y) {
		_currentState.mouseDown(x, y);
	}

	public void mouseUp(int x, int y) {
		_currentState.mouseUp(x, y);
	}

	public void mouseDrag(int x, int y) {
		_currentState.mouseDrag(x, y);
	}

	public void select(Drawing drawing, Color color) {
		_currentState.select(drawing, color);
	}
}
