package state.jcooper;


import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import jcooper_support.ICommand;
import jcooper_support.JxFrame;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 */
public class Window4State extends JxFrame implements ActionListener {
	private static final long serialVersionUID = 0L;

	private JToolBar _toolbar;
	private Mediator _mediator;

	public Window4State() {
		super("State Drawing");
		JPanel jp = new JPanel();
		getContentPane().add(jp);
		_mediator = new Mediator();
		jp.setLayout(new BorderLayout());
		_toolbar = new JToolBar();
		_toolbar.setFloatable(false);
		jp.add("North", _toolbar);
		PickButton pick = new PickButton(this, _mediator);
		_toolbar.add(pick);
		_toolbar.addSeparator();
		RectButton rect = new RectButton(this, _mediator);
		_toolbar.add(rect);
		FillButton fill = new FillButton(this, _mediator);
		_toolbar.add(fill);
		CircleButton circ = new CircleButton(this, _mediator);
		_toolbar.add(circ);
		_toolbar.addSeparator();
		ClearButton clr = new ClearButton(this, _mediator);
		_toolbar.add(clr);
		JCanvas canvas = new JCanvas(_mediator);
		jp.add("Center", canvas);
		MouseApp mouseApp = new MouseApp(_mediator);
		canvas.addMouseListener(mouseApp);
		MouseMoveApp mouseMoveApp = new MouseMoveApp(_mediator);
		canvas.addMouseMotionListener(mouseMoveApp);
		setSize(new Dimension(400, 300));
		setVisible(true);
	}

	public void actionPerformed(ActionEvent e) {
		ICommand comd = (ICommand) e.getSource();
		comd.execute();
	}
}

class MouseApp extends MouseAdapter {
	private Mediator _mediator;

	public MouseApp(Mediator mediator) {
		super();
		_mediator = mediator;
	}

	public void mousePressed(MouseEvent e) {
		_mediator.mouseDown(e.getX(), e.getY());
	}

	public void mouseReleased(MouseEvent e) {
		_mediator.mouseUp(e.getX(), e.getY());
	}
}

class MouseMoveApp extends MouseMotionAdapter {
	private Mediator _mediator;

	public MouseMoveApp(Mediator md) {
		super();
		_mediator = md;
	}

	public void mouseDragged(MouseEvent e) {
		_mediator.mouseDrag(e.getX(), e.getY());
	}
}
