package strategy;

/**
 * Implements the the abstract Strategy design pattern protocol
 * 
 * Defines the Strategy and Context role. Also provides methods
 * for setting the Strategy for a given Context.
 * 
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class StrategyProtocol {

	/**
	 * Defines the Strategy role
	 */
	protected abstract class Strategy {
	}

	/**
	 * Defines the Context role
	 */
	protected abstract class Context {
		public Strategy concreteStrategy;

		public void setConcreteStrategy(Strategy concreteStrategy) {
			this.concreteStrategy = concreteStrategy;
		}
	}

	/**
	 * Sets the strategy for a given context
	 * 
	 * @param c	the context to set the strategy for
	 * @param s	the new strategy
	 */
	public void setConcreteStrategy(Context c, Strategy s) {
		c.setConcreteStrategy(s);
	}
}