package strategy.hk;

/**
 * Implements the driver for the strategy design pattern example.
 * 
 * Intent: Define a family of algorithms, encapsulate each one, and make them
 * interchangeable. Strategy lets the algorithm vary independently from clients
 * that use it.
 * 
 * Participating objects are LinearSort and BubbleSort as Strategies, and Sorter
 * as Context.
 * 
 * In this example, an array of 10 numbers is to be sorted. Depending on the
 * number of arguments of the call to Main, linear sort or bubblesort are used
 * as sorting algorithms.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {

	/**
	 * Returns the content of the int array in a string
	 * 
	 * @param numbers
	 *            the int array to display
	 * @returns a string with all the ints from the array
	 */
	private static String show(int[] numbers) {
		String out = "";
		for (int i = 0; i < numbers.length; i++) {
			out += (numbers[i] + " ");
		}
		return out;
	}

	/**
	 * Implements the driver for the strategy example. If called with more than
	 * zero arguments, bubblesort is used to sort the array of ten numbers;
	 * otherwise linear sort.
	 */
	public static void main(String[] args) {
		int[] numbers = { 3, 2, 9, 8, 1, 5, 6, 4, 7, 0 };

		LinearSort sort1 = new LinearSort();
		BubbleSort sort2 = new BubbleSort();

		Sorter sorter = new Sorter(numbers);
		SortingStrategyTeam strategyTeam = new SortingStrategyTeam();
		strategyTeam.activate();

		int selection = (int) Math.round(Math.random()); //either 0 or 1
		if (selection /*args.length*/ == 1)
			strategyTeam.setConcreteStrategy(sorter, sort1);
		else
			strategyTeam.setConcreteStrategy(sorter, sort2);

		System.out.println("\nPreparing sort...");
		System.out.println("original: " + sorter.show());
		sorter.sort();
		System.out.println("sorted:   " + sorter.show());
	}
}
