package strategy.hk;

import strategy.StrategyProtocol;

/**
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class SortingStrategyTeam extends StrategyProtocol {

	/* Role classes extending role class Strategy
	 * which represent each role played by each sorting strategy,
	 * to be lifted to in setConcreteStrategy!
	 */
	protected abstract class Strategy {
		public abstract void sort(int[] numbers);
	}

	protected class LinearSortRole extends Strategy playedBy LinearSort {
		sort -> sort;
	}

	protected class BubbleSortRole extends Strategy playedBy BubbleSort {
		sort -> sort;
	}

	protected class Context playedBy Sorter {
		callin void sort(){
			this.concreteStrategy.sort(getNumbers());
		}
		sort <- replace sort;
		public abstract int[] getNumbers();
		getNumbers -> get _numbers;
	}

 	public <AnyBase base Strategy> void setConcreteStrategy(Sorter as Context c, AnyBase as Strategy s) {
 		super.setConcreteStrategy(c, s);
 	}
}
