/*
   2009/09/03
*/
package templateMethod.hk;

/**
 * Defines the interface for generating decorated strings. 
 * 
 * The template method is the team method <code>generate(String)</code>,
 * which uses all other methods defined in the base classes.
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 * @author Miguel P. Monteiro - adapted to Object Teams/Java
 */
public team class TemplateTeam {

	public abstract class GeneratorRole {
	    /**
	     * Decorates a string. This is the <i>TemplateMethod()</i>.
	     *
	     * @param s the string to decorate
	     * @return the decorated string
	     */
		public String templateMethodGenerate(String s) {
			s = _generatorStrategy.prepare(s);
			s = _generatorStrategy.filter(s);
			s = _generatorStrategy.finalize(s);
			return s;
		}

	    /**
	     * Prepares a string for decoration.
	     *
	     * @param s the string to filter
	     * @return the prepared string
	     */
		public abstract String prepare(String s);

	    /**
	     * Filters a string. 
	     *
	     * @param s the string to filter
	     * @return the filtered string
	     */
		public abstract String filter(String s);

	    /**
	     * Finalizes a string. This is the last step in the template method.
	     *
	     * @param s the string to finalize
	     * @return the finalized string
	     */
		public abstract String finalize(String s);
	}

	public class SimpleGeneratorRole extends GeneratorRole playedBy SimpleGenerator {
		prepare -> prepare;
		filter -> filter;
		String finalize(String s) -> String finalize(String s);
	}
	public class FancyGeneratorRole extends GeneratorRole playedBy FancyGenerator {
		prepare -> prepare;
		filter -> filter;
		String finalize(String s) -> String finalize(String s);		
	}

	public GeneratorRole _generatorStrategy;

	public TemplateTeam(SimpleGenerator as SimpleGeneratorRole simple){
		this._generatorStrategy = simple;
	}
	public TemplateTeam(FancyGenerator as FancyGeneratorRole fancy) {
		this._generatorStrategy = fancy;
	}

	public String generate(String s) {
		//Forwards to the template method
		return _generatorStrategy.templateMethodGenerate(s);
	}
}
