package templateMethod.jcooper;

import java.awt.*;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public team class TriangleTeam {

	public class TriangleRole playedBy Triangle {
		Point getP1() -> Point getP1();

		Point getP2() -> Point getP2();

		Point getP3() -> Point getP3();

		//Bindings for this calin method are made in sub-roles
		callin void drawTriangle(Graphics g) {
			_triangleStrategy = this;
			TriangleTeam.this.draw(g);
		}
		drawTriangle <- replace draw;

		public abstract Point draw2ndLine(Graphics g, Point a, Point b);
		draw2ndLine -> draw2ndLine;
	}

	public class StandardTriangleRole extends TriangleRole playedBy StandardTriangle {
	}

	public class IsocelesTriangleRole extends TriangleRole playedBy IsocelesTriangle {
	}

	/******************************************************************/
	private TriangleRole _triangleStrategy;

	public void drawLine(Graphics g, Point a, Point b) {
		g.drawLine(a.x, a.y, b.x, b.y);
	}
	public void closeTriangle(Graphics g, Point c) {
		// draw back to first point
		g.drawLine(c.x, c.y, _triangleStrategy.getP1().x, _triangleStrategy.getP1().y);
	}

	//Template method
	public void draw(Graphics g) {
		System.out.println("Team method");
		// This routine draws a general triangle
		drawLine(g, _triangleStrategy.getP1(), _triangleStrategy.getP2());
		Point current = _triangleStrategy.draw2ndLine(g, _triangleStrategy.getP2(), _triangleStrategy.getP3());
		closeTriangle(g, current);
	}
}
