package visitor;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public abstract team class VisitorProtocol {

	public abstract class VisitableNode {
		/**
		 * Declares accept(..) for visitable nodes
		 * 
		 * @param visitor the visitor that is to be accepted
		 */
		public void accept(Visitor visitor) {
		}
	}

	protected class Node extends VisitableNode {
		/**
		 * Declares accept(..) for regular nodes
		 * 
		 * @param visitor the visitor that is to be accepted
		 */
		public void accept(Visitor visitor) {
			visitor.visitNode(this);
		}
	}

	protected abstract class Leaf extends VisitableNode {
		/**
		 * Declares accept(..) for leaf nodes
		 * 
		 * @param visitor the visitor that is to be accepted
		 */
		public void accept(Visitor visitor) {
			visitor.visitLeaf(this);
		}

	}

	public abstract class Visitor {
//		 To verify Family Polymorphism; just remove the comments from this constructor
//		 public Visitor(String message) {
//			 System.err.println(message);
//		 }

		/**
		 * Defines a method signature for visiting regular nodes.
		 * 
		 * @param node the regular node to visit
		 */
		public abstract void visitNode(VisitableNode node);

		/**
		 * Defines a method signature for visiting leaf nodes.
		 * 
		 * @param node t he leaf node to visit
		 */
		public abstract void visitLeaf(VisitableNode node);

		/**
		 * Defines a method signature for returning the visitor's results
		 * 
		 * @param node a string containig the visitor's results
		 */
		public abstract String report();
	}

	public void acceptVisit(VisitableNode node) {
		Visitor visitor = new Visitor();
		node.accept(visitor);
	}

}