package visitor.hk1;
/**
 * Implements the driver for the Visitor design pattern example.
 * 
 * Intent: Represents an operation to be performed on the elements of an
 * object structure. Visitor lets you define a new operation without changing
 * the classes of the elements on which it operates
 * 
 * Participating classes are SummationVisitorTeam and
 * TraversalVisitorTeam as ConcreteVisitors.
 * 
 * BinaryTreeNode and BinaryTreeLeaf are
 * ConcreteElements.
 * 
 * In this example, a binary tree is built that has int values as leafs.
 * SummationVisitorTeam is a Visitor that collects the sum
 * of the leaf values (should be 6).
 * 
 * TraversalVisitorTeam is a visitor that collects a description of the tree
 * like {{1,2},3}
 * 
 * @author Jan Hannemann, Gregor Kiczales - original Java/AspectJ example
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {

	/**
	 * Implements the driver for the Visitor design pattern example.
	 * 
	 * 
	 * @param args
	 *            the command-line parameters, unused
	 */
	public static void main(String[] args) {
		System.out.println("Building the tree (1): leaves");

		BinaryTreeLeaf one = new BinaryTreeLeaf(1);
		BinaryTreeLeaf two = new BinaryTreeLeaf(2);
		BinaryTreeLeaf three = new BinaryTreeLeaf(3);

		System.out.println("Building the tree (1): regular nodes");

		BinaryTreeNode regN = new BinaryTreeNode(one, two);
		BinaryTreeNode root = new BinaryTreeNode(regN, three);

		System.out.println("The tree now looks like this: ");
		System.out.println("         regN                 ");
		System.out.println("        /    \\               ");
		System.out.println("    regN      3               ");
		System.out.println("   /    \\                    ");
		System.out.println("  1      2                    ");

		System.out.println("Visitor 1: GeneralVisitor, collects the sum of leaf");
		System.out.println("values. Result should be 6.");

		SummationVisitorTeam sumVisitor = new SummationVisitorTeam();
		sumVisitor.activate();
		sumVisitor.acceptVisit(root);
		System.out.println(sumVisitor.visitorReport());

		System.out.println("Visitor 2: TraversalVisitor, collects a tree");
		System.out.println("representation. Result should be {{1,2},3}.");

		TraversalVisitorTeam traversalVisitor = new TraversalVisitorTeam();
		traversalVisitor.activate();
		traversalVisitor.acceptVisit(root);
		System.out.println(traversalVisitor.visitorReport());
	}
}
