/*
 * Created on 7/Jul/2004
 */
package visitor.jcooper1;

/**
 * @author James Cooper, adapted and refactored by Miguel P. Monteiro
 * @author Joao Gomes - adapted to Object Teams/Java
 */
public class Main {
	private static void printHelpMessage() {
		System.out.println("Design pattern Template Method");
		System.out.println();
		System.out.println("Intent: Represent an operation to be performed on the elements");
		System.out.println("of an object structure. Visitor lets you define a new operation");
		System.out.println("without changing the classes of the elements on which it operates.");
		System.out.println();
		System.out.println("This example is based on a subset of the scenario used for Composite.");
		System.out.println("There are two Concrete Element participants: Employee and Boss.");
		System.out.println("Bosses are Employees, too: class Boss extends class Employee.");
		System.out.println("The idea is to report the number of vacation days that employees have taken so far this year.");
		System.out.println("There are two visitors:");
		System.out.println("1) team OldEmployeeVisitor just sums the vacation data for all employees.");
		System.out.println("2) team OldBossVisitor also takes into account Bosses' bonus days.");
		System.out.println();
		System.out.println("In the window that appears, the two lines of displayed data represent");
		System.out.println("the two sums that are computed when the user clicks on the Vacations button.");
	}

	private static Employee[] createEmployees() {
		Employee[] employees = new Employee[7];
		int i = 0;
		employees[i++] = new Employee("Susan Bear", 55000, 12, 1);
		employees[i++] = new Employee("Adam Gehr", 150000, 9, 0);
		employees[i++] = new Employee("Fred Harris", 50000, 15, 2);
		employees[i++] = new Employee("David Oakley", 57000, 12, 2);
		employees[i++] = new Employee("Larry Thomas", 100000, 20, 6);
		Boss boss1 = new Boss("Leslie Susi", 175000, 16, 4);
		boss1.setBonusDays(12);
		Boss boss2 = new Boss("Laurence Byerly", 35000, 17, 6);
		boss2.setBonusDays(17);
		employees[i++] = boss1;
		employees[i++] = boss2;
		return employees;
	}

	public static void main(String[] args) {
		printHelpMessage();
		VacationVisitorTeam vacationVisitorTeam = new VacationVisitorTeam();
		BossVacationVisitorTeam bossVacVisitorTeam = new BossVacationVisitorTeam();
		new Window4Visitor(createEmployees(), vacationVisitorTeam, bossVacVisitorTeam);
	}
}
